@@ -24,14 +24,14 @@ from optax._src import combine
 from optax._src import numerics
 from optax._src import update
 from optax.contrib import _sam
-from optax.tree_utils import _state_utils
+import optax.tree
 
 _BASE_OPTIMIZERS_UNDER_TEST = [
-    dict(base_opt_name='sgd', base_opt_kwargs=dict(learning_rate=1e-3)),
+    {'base_opt_name': 'sgd', 'base_opt_kwargs': {'learning_rate': 1e-3}},
 ]
 _ADVERSARIAL_OPTIMIZERS_UNDER_TEST = [
-    dict(adv_opt_name='sgd', adv_opt_kwargs=dict(learning_rate=1e-5)),
-    dict(adv_opt_name='adam', adv_opt_kwargs=dict(learning_rate=1e-4)),
+    {'adv_opt_name': 'sgd', 'adv_opt_kwargs': {'learning_rate': 1e-5}},
+    {'adv_opt_name': 'adam', 'adv_opt_kwargs': {'learning_rate': 1e-4}},
 ]
 
 
@@ -79,7 +79,7 @@ class SAMTest(chex.TestCase):
     initial_params, final_params, get_updates = target(dtype)
 
     if opaque_mode:
-      update_kwargs = dict(grad_fn=lambda p, _: get_updates(p))
+      update_kwargs = {'grad_fn': lambda p, _: get_updates(p)}
     else:
       update_kwargs = {}
 
@@ -93,7 +93,7 @@ class SAMTest(chex.TestCase):
     params = initial_params
     state = opt.init(params)
     # A no-op change, to verify that tree map works.
-    state = _state_utils.tree_map_params(opt, lambda v: v, state)
+    state = optax.tree.map_params(opt, lambda v: v, state)
 
     for _ in range(25000 * sync_period):
       params, state = step(params, state)
