@@ -15,14 +15,34 @@
 """Classification losses."""
 
 import functools
+import operator
 from typing import Optional, Union
 
 import chex
 import jax
 import jax.numpy as jnp
+import jax.scipy.special
 from optax import projections
 
 
+def canonicalize_axis(axis, ndim):
+  """Vendored version of :func:`numpy.lib.array_utils.normalize_axis_index`.
+  """
+  if 0 <= (axis := operator.index(axis)) < ndim:
+    return axis
+  elif -ndim <= axis < 0:
+    return axis + ndim
+  else:
+    raise ValueError(f'axis {axis} is out of bounds for array of '
+                     f'dimension {ndim}')
+
+
+def canonicalize_axes(axes, ndim) -> tuple[int, ...]:
+  """Vendored version of :func:`numpy.lib.array_utils.normalize_axis_tuple`.
+  """
+  return tuple(canonicalize_axis(x, ndim) for x in axes)
+
+
 def sigmoid_binary_cross_entropy(
     logits,
     labels,
@@ -55,7 +75,7 @@ def sigmoid_binary_cross_entropy(
     <http://www.deeplearningbook.org/contents/prob.html>`_, 2016
   """
   chex.assert_type([logits], float)
-  labels = labels.astype(logits.dtype)
+  labels = jnp.astype(labels, logits.dtype)
   log_p = jax.nn.log_sigmoid(logits)
   # log(1 - sigmoid(x)) = log_sigmoid(-x), the latter more numerically stable
   log_not_p = jax.nn.log_sigmoid(-logits)
@@ -232,7 +252,8 @@ def softmax_cross_entropy(
     labels: One-hot encoded labels, with shape `[batch_size, num_classes]`. Each
       row represents the true class distribution for a single example.
     axis: Axis or axes along which to compute.
-    where: Elements to include in the computation.
+    where: Elements to include in the computation of shape ``[batch_size]`` or
+      logits.shape.
 
   Returns:
     Cross-entropy between each prediction and the corresponding target
@@ -241,11 +262,12 @@ def softmax_cross_entropy(
   Examples:
     >>> import optax
     >>> import jax.numpy as jnp
+    >>> jnp.set_printoptions(precision=4)
     >>> # example: batch_size = 2, num_classes = 3
     >>> logits = jnp.array([[1.2, -0.8, -0.5], [0.9, -1.2, 1.1]])
     >>> labels = jnp.array([[1.0, 0.0, 0.0], [0.0, 1.0, 0.0]])
     >>> print(optax.softmax_cross_entropy(logits, labels))
-    [0.2761297 2.951799 ]
+    [0.2761 2.9518]
 
   References:
     `Cross-entropy Loss <https://en.wikipedia.org/wiki/Cross-entropy>`_,
@@ -266,6 +288,8 @@ def softmax_cross_entropy(
     Added ``axis`` and ``where`` arguments.
   """
   chex.assert_type([logits], float)
+  if where is not None and where.ndim != logits.ndim:
+    where = jnp.expand_dims(where, axis)
   log_probs = jax.nn.log_softmax(logits, axis, where)
   return -(labels * log_probs).sum(axis, where=where)
 
@@ -273,7 +297,7 @@ def softmax_cross_entropy(
 def softmax_cross_entropy_with_integer_labels(
     logits: chex.Array,
     labels: chex.Array,
-    axis: Union[int, tuple[int, ...], None] = -1,
+    axis: Union[int, tuple[int, ...]] = -1,
     where: Union[chex.Array, None] = None,
 ) -> chex.Array:
   r"""Computes softmax cross entropy between the logits and integer labels.
@@ -297,8 +321,12 @@ def softmax_cross_entropy_with_integer_labels(
     labels: Integers specifying the correct class for each input, with shape
       ``[batch_size]``. Class labels are assumed to be between 0 and
       ``num_classes - 1`` inclusive.
-    axis: Axis or axes along which to compute.
-    where: Elements to include in the computation.
+    axis: Axis or axes along which to compute. If a tuple of axes is passed
+      then ``num_classes`` must match the total number of elements in ``axis``
+      dimensions and a label is interpreted as a flat index in a ``logits``
+      slice of shape ``logits[axis]``.
+    where: Elements to include in the computation of shape ``[batch_size]``
+      or logits.shape.
 
   Returns:
     Cross-entropy between each prediction and the corresponding target
@@ -307,11 +335,28 @@ def softmax_cross_entropy_with_integer_labels(
   Examples:
     >>> import optax
     >>> import jax.numpy as jnp
+    >>> jnp.set_printoptions(precision=4)
     >>> # example: batch_size = 2, num_classes = 3
     >>> logits = jnp.array([[1.2, -0.8, -0.5], [0.9, -1.2, 1.1]])
     >>> labels = jnp.array([0, 1])
     >>> print(optax.softmax_cross_entropy_with_integer_labels(logits, labels))
-    [0.2761297 2.951799 ]
+    [0.2761 2.9518]
+
+    >>> import jax.numpy as jnp
+    >>> import numpy as np
+    >>> import optax
+    >>> jnp.set_printoptions(precision=4)
+    >>> # example: batch_size = (1, 2), num_classes = 12 (i.e. 3 * 4)
+    >>> shape = (1, 2, 3, 4)
+    >>> logits = jnp.arange(np.prod(shape), dtype=jnp.float32).reshape(shape)
+    >>> # elements indices in slice of shape (3, 4)
+    >>> ix = jnp.array([[1, 2]])
+    >>> jx = jnp.array([[1, 3]])
+    >>> labels = jnp.ravel_multi_index((ix, jx), shape[2:])
+    >>> cross_entropy = optax.softmax_cross_entropy_with_integer_labels(
+    ...     logits, labels, axis=(2, 3))
+    >>> print(cross_entropy)
+    [[6.4587 0.4587]]
 
   References:
     `Cross-entropy Loss <https://en.wikipedia.org/wiki/Cross-entropy>`_,
@@ -329,18 +374,35 @@ def softmax_cross_entropy_with_integer_labels(
   """
   chex.assert_type([logits], float)
   chex.assert_type([labels], int)
+  if where is not None and where.ndim != logits.ndim:
+    where = jnp.expand_dims(where, axis)
+  if isinstance(axis, int):
+    axis = canonicalize_axis(axis, logits.ndim)
+  elif isinstance(axis, tuple):
+    # Move all "feature" dimensions to the end preserving axis ordering and
+    # subsequent flattening "feature" dimensions to a single one.
+    logit_axis = canonicalize_axes(axis, logits.ndim)
+    batch_axis = tuple(x for x in range(logits.ndim) if x not in logit_axis)
+    axis = len(batch_axis)
+    logits = logits.transpose(batch_axis + logit_axis)
+    logits = logits.reshape(logits.shape[:len(batch_axis)] + (-1,))
+    if where is not None:
+      where = where.transpose(batch_axis + logit_axis)
+      where = where.reshape(where.shape[:len(batch_axis)] + (-1,))
+  else:
+    raise ValueError('Keyword argument \'axis\' must be of type \'int\' or '
+                     f'\'tuple[int, ...]\' but actual type is {type(axis)}.')
   # This is like jnp.take_along_axis(jax.nn.log_softmax(...), ...) except that
   # we avoid subtracting the normalizer from all values, just from the values
   # for the correct labels.
-  logits_max = jnp.max(
-      logits, axis, keepdims=True, where=where, initial=-jnp.inf
-  )
-  logits -= jax.lax.stop_gradient(logits_max)
   label_logits = jnp.take_along_axis(
       logits, jnp.expand_dims(labels, axis), axis=axis
   ).take(0, axis=axis)
-  log_normalizers = jnp.log(jnp.sum(jnp.exp(logits), axis=axis, where=where))
-  return log_normalizers - label_logits
+  log_normalizers = jax.nn.logsumexp(logits, axis=axis, where=where)
+  out = log_normalizers - label_logits
+  if where is not None:
+    out = jnp.where(jnp.any(where, axis), out, 0.0)
+  return out
 
 
 @functools.partial(
@@ -436,7 +498,7 @@ def poly_loss_cross_entropy(
       - For the ImageNet 2d image classification, epsilon = 2.0.
       - For the 2d Instance Segmentation and object detection, epsilon = -1.0.
       - It is also recommended to adjust this value based on the task, e.g. by
-        using grid search.
+      using grid search.
     axis: Axis or axes along which to compute.
     where: Elements to include in the computation.
 
@@ -753,50 +815,85 @@ def sigmoid_focal_loss(
     alpha: Optional[float] = None,
     gamma: float = 2.0,
 ) -> chex.Array:
-  """Sigmoid focal loss.
+  r"""Sigmoid focal loss with numerical stability improvements.
 
-  The focal loss is a re-weighted cross entropy for unbalanced problems.
-  Use this loss function if classes are not mutually exclusive.
-  See `sigmoid_binary_cross_entropy` for more information.
+  The focal loss is a dynamically scaled cross entropy loss, where the scaling
+  factor decays to zero as confidence in the correct class increases. This
+  addresses class imbalance by down-weighting easy examples and focusing on
+  hard examples.
+
+  This implementation uses log-space computation for the focal weight
+  :math:`(1-p_t)^\gamma` to ensure numerical stability, especially for
+  :math:`\gamma < 2` and extreme logit values.
+
+  The loss is defined as:
+
+  .. math::
+    FL(p_t) = -\alpha_t (1-p_t)^\gamma \log(p_t)
+
+  where :math:`p_t` is the predicted probability of the correct class:
+
+  .. math::
+    p_t = \begin{cases}
+      p & \text{if } y = 1 \\
+      1-p & \text{if } y = 0
+    \end{cases}
+
+  and :math:`\alpha_t` is the weighting factor:
+
+  .. math::
+    \alpha_t = \begin{cases}
+      \alpha & \text{if } y = 1 \\
+      1-\alpha & \text{if } y = 0
+    \end{cases}
 
   Args:
-    logits: Array of floats. The predictions for each example. The predictions
-      for each example.
-    labels: Array of floats. Labels and logits must have the same shape. The
-      label array must contain the binary classification labels for each element
-      in the data set (0 for the out-of-class and 1 for in-class).
-    alpha: (optional) Weighting factor in range (0,1) to balance positive vs
-      negative examples. Default None (no weighting).
-    gamma: Exponent of the modulating factor (1 - p_t). Balances easy vs hard
-      examples.
+    logits: Array of unnormalized log probabilities, with shape `[..., ]`.
+      The predictions for each example.
+    labels: Array of labels with shape broadcastable to `logits`. Can be:
+      - Binary labels `{0, 1}` for binary classification
+      - Continuous labels `[0, 1]` for soft targets or label smoothing
+    alpha: (optional) Weighting factor in range `(0, 1)` to balance positive vs
+      negative examples. Default `None` (no weighting).
+    gamma: Exponent of the modulating factor `(1 - p_t)`. Higher values focus
+      more on hard examples. Default `2.0`.
 
   Returns:
-    A loss value array with a shape identical to the logits and target
-    arrays.
+    Focal loss values with shape identical to `logits`.
 
   References:
-    Lin et al. `Focal Loss for Dense Object Detection
+    Lin et al, `Focal Loss for Dense Object Detection
     <https://arxiv.org/abs/1708.02002>`_, 2017
+
+  .. versionchanged:: 0.2.5
+    Added numerical stability improvements using log-space computation.
+    Added support for continuous labels in `[0, 1]`.
   """
-  alpha = -1 if alpha is None else alpha
-
   chex.assert_type([logits], float)
-  labels = labels.astype(logits.dtype)
-  # see also the original paper's implementation at:
-  # https://github.com/facebookresearch/fvcore/blob/main/fvcore/nn/focal_loss.py
-  p = jax.nn.sigmoid(logits)
+  labels = jnp.astype(labels, logits.dtype)
+
+  # Cross-entropy loss
   ce_loss = sigmoid_binary_cross_entropy(logits, labels)
-  p_t = p * labels + (1 - p) * (1 - labels)
-  loss = ce_loss * ((1 - p_t) ** gamma)
 
-  weighted = (
-      lambda loss_arg: (alpha * labels + (1 - alpha) * (1 - labels)) * loss_arg
+  # Compute log(1-p_t) using logsumexp unconditionally
+  log_p = jax.nn.log_sigmoid(logits)
+  log_q = jax.nn.log_sigmoid(-logits)
+
+  log_one_minus_p_t = jax.scipy.special.logsumexp(
+      jnp.stack([log_p, log_q], axis=-1),
+      axis=-1,
+      b=jnp.stack([1 - labels, labels], axis=-1)
   )
-  not_weighted = lambda loss_arg: loss_arg
 
-  loss = jax.lax.cond(alpha >= 0, weighted, not_weighted, loss)
+  # Focal weight and final loss
+  focal_weight = jnp.exp(gamma * log_one_minus_p_t)
+  loss = ce_loss * focal_weight
 
-  return loss
+  # Alpha weighting
+  if alpha is None:
+    return loss
+  weighted = (alpha * labels + (1.0 - alpha) * (1.0 - labels)) * loss
+  return weighted
 
 
 def _multiclass_sparsemax_loss(
