@@ -31,27 +31,11 @@ class SoftmaxCrossEntropyTest(parameterized.TestCase):
 
   def setUp(self):
     super().setUp()
-    self.ys = np.array(
-        [
-            [10.0, 1.0, -2.0],
-            [1.0, 4.0, 0.2],
-        ],
-        dtype=np.float32,
-    )
-    self.ts = np.array(
-        [
-            [0.0, 1.0, 0.0],
-            [1.0, 0.0, 0.0],
-        ],
-        dtype=np.float32,
-    )
-    self.exp = np.array(
-        [
-            9.00013,
-            3.0696733,
-        ],
-        dtype=np.float32,
-    )
+    self.ys = np.array([[10.0, 1.0, -2.0], [1.0, 4.0, 0.2]], dtype=np.float32)
+    self.ts = np.array([[0.0, 1.0, 0.0], [1.0, 0.0, 0.0]], dtype=np.float32)
+    self.exp = np.array([9.00013, 3.0696733], dtype=np.float32)
+    self.mask = np.array([True, False])
+    self.per_logit_mask = np.array([[1, 0, 0], [0, 0, 0]]).astype(bool)
 
   @chex.all_variants
   def test_scalar(self):
@@ -73,6 +57,13 @@ class SoftmaxCrossEntropyTest(parameterized.TestCase):
         atol=1e-4,
     )
 
+  def test_per_logit_mask(self):
+    """Tests for a full batch."""
+    where = self.per_logit_mask
+    out = _classification.softmax_cross_entropy(self.ys, self.ts, where=where)
+    # Check that outputs where all logits are masked are 0.0.
+    np.testing.assert_array_equal(jnp.where(jnp.any(where, -1), 0.0, out), 0)
+
   def test_gradient(self):
     """Tests gradient ok."""
     jaxtest.check_grads(
@@ -80,8 +71,13 @@ class SoftmaxCrossEntropyTest(parameterized.TestCase):
         (self.ys[:2], self.ts[:2]),
         order=1,
     )
+    # check if the gradients are finite when a mask is present
+    loss_fn = lambda *args, **kw: jnp.sum(
+        _classification.softmax_cross_entropy(*args, **kw))
+    gs = jax.grad(loss_fn)(self.ys, labels=self.ts, where=self.mask)
+    self.assertTrue(jnp.all(jnp.isfinite(gs)))
 
-  @parameterized.parameters(dict(size=5), dict(size=10))
+  @parameterized.parameters({'size': 5}, {'size': 10})
   def test_mask(self, size):
     preds = np.random.normal(size=size)
     targets = np.random.dirichlet(np.ones(size))
@@ -91,9 +87,9 @@ class SoftmaxCrossEntropyTest(parameterized.TestCase):
     np.testing.assert_allclose(x, y, atol=1e-4)
 
   @parameterized.parameters(
-      dict(axis=0, shape=[4, 5, 6]),
-      dict(axis=1, shape=[4, 5, 6]),
-      dict(axis=2, shape=[4, 5, 6]),
+      {'axis': 0, 'shape': [4, 5, 6]},
+      {'axis': 1, 'shape': [4, 5, 6]},
+      {'axis': 2, 'shape': [4, 5, 6]},
   )
   def test_axis(self, shape, axis):
     preds = np.random.normal(size=shape)
@@ -189,6 +185,8 @@ class SoftmaxCrossEntropyWithIntegerLabelsTest(parameterized.TestCase):
     super().setUp()
     self.ys = np.array([[10.0, 1.0, -2.0], [1.0, 4.0, 0.2]], dtype=np.float32)
     self.ts = np.array([1, 0], dtype=np.int32)
+    self.mask = np.array([True, False])
+    self.per_logit_mask = np.array([[1, 0, 0], [0, 0, 0]]).astype(bool)
 
   @chex.all_variants
   def test_consistent_with_softmax_cross_entropy_scalar(self):
@@ -228,11 +226,24 @@ class SoftmaxCrossEntropyWithIntegerLabelsTest(parameterized.TestCase):
         (self.ys,),
         order=1,
     )
+    # check if the gradients are finite when a mask is present
+    loss_fn = lambda *args, **kw: jnp.sum(
+        _classification.softmax_cross_entropy_with_integer_labels(*args, **kw))
+    gs = jax.grad(loss_fn)(self.ys, labels=self.ts, where=self.mask)
+    self.assertTrue(jnp.all(jnp.isfinite(gs)))
+
+  def test_per_logit_mask(self):
+    """Tests for a full batch."""
+    where = self.per_logit_mask
+    out = _classification.softmax_cross_entropy_with_integer_labels(
+        self.ys, self.ts, where=where)
+    # Check that outputs where all logits are masked are 0.0.
+    np.testing.assert_array_equal(jnp.where(jnp.any(where, -1), 0.0, out), 0)
 
   @parameterized.parameters(
-      dict(axis=0, shape=[4, 5, 6]),
-      dict(axis=1, shape=[4, 5, 6]),
-      dict(axis=2, shape=[4, 5, 6]),
+      {'axis': 0, 'shape': [4, 5, 6]},
+      {'axis': 1, 'shape': [4, 5, 6]},
+      {'axis': 2, 'shape': [4, 5, 6]},
   )
   def test_axis(self, shape, axis):
     preds = np.random.normal(size=shape)
@@ -247,62 +258,98 @@ class SoftmaxCrossEntropyWithIntegerLabelsTest(parameterized.TestCase):
     )
     np.testing.assert_allclose(x, y, atol=1e-4)
 
+  @parameterized.parameters(
+      {'axis': (1, 3), 'shape': (2, 3, 4, 5)},
+      {'axis': (3, 2), 'shape': (2, 3, 4, 5)},
+      {'axis': (2, 3), 'shape': (2, 3, 4, 5)},
+      {'axis': (-3, -1), 'shape': (2, 3, 4, 5)},
+      {'axis': (-1, -2), 'shape': (2, 3, 4, 5)},
+      {'axis': (-2, -1), 'shape': (2, 3, 4, 5)},
+      {'axis': (0, 1, 3), 'shape': (2, 3, 4, 5)},
+      {'axis': (-4, -3, -1), 'shape': (2, 3, 4, 5)},
+  )
+  def test_axes(self, shape: tuple[int, ...], axis: tuple[int, ...]):
+    # Canonicalize axis and calculate shapes.
+    ndim = len(shape)
+    logits_axis = tuple((x + ndim) % ndim for x in axis)
+    labels_axis = tuple(x for x in range(ndim) if x not in logits_axis)
+    # Obtain shapes of batch and logits subspaces.
+    logits_shape = tuple(shape[x] for x in logits_axis)
+    labels_shape = tuple(shape[x] for x in labels_axis)
+    num_classes: float = np.prod(logits_shape).item()
+
+    key = jax.random.key(42)
+    keys = jax.random.split(key, 2)
+    logits = jax.random.uniform(keys[0], labels_shape + (num_classes,))
+    labels = jax.random.randint(keys[1], labels_shape, 0, num_classes - 1)
+
+    fn = _classification.softmax_cross_entropy_with_integer_labels
+    desired = fn(logits, labels)
+
+    # Apply inverse axes permutation to obtain an array of `shape` shape.
+    perm = labels_axis + logits_axis
+    perm_inv = tuple(i for i, _ in sorted(enumerate(perm), key=lambda x: x[1]))
+    logits = logits.reshape(labels_shape + logits_shape).transpose(perm_inv)
+    assert logits.shape == shape
+    actual = fn(logits, labels, axis)
+    np.testing.assert_allclose(actual, desired)
+
 
 class SigmoidCrossEntropyTest(parameterized.TestCase):
 
   @parameterized.parameters(
-      dict(
-          preds=np.array([-1e09, -1e-09]),
-          labels=np.array([1.0, 0.0]),
-          expected=5e08,
-      ),
-      dict(
-          preds=np.array([-1e09, -1e-09]),
-          labels=np.array([0.0, 1.0]),
-          expected=0.3465736,
-      ),
-      dict(
-          preds=np.array([1e09, 1e-09]),
-          labels=np.array([1.0, 0.0]),
-          expected=0.3465736,
-      ),
-      dict(
-          preds=np.array([1e09, 1e-09]),
-          labels=np.array([0.0, 1.0]),
-          expected=5e08,
-      ),
-      dict(
-          preds=np.array([-1e09, 1e-09]),
-          labels=np.array([1.0, 0.0]),
-          expected=5e08,
-      ),
-      dict(
-          preds=np.array([-1e09, 1e-09]),
-          labels=np.array([0.0, 1.0]),
-          expected=0.3465736,
-      ),
-      dict(
-          preds=np.array([1e09, -1e-09]),
-          labels=np.array([1.0, 0.0]),
-          expected=0.3465736,
-      ),
-      dict(
-          preds=np.array([1e09, -1e-09]),
-          labels=np.array([0.0, 1.0]),
-          expected=5e08,
-      ),
-      dict(
-          preds=np.array([0.0, 0.0]),
-          labels=np.array([1.0, 0.0]),
-          expected=0.6931472,
-      ),
-      dict(
-          preds=np.array([0.0, 0.0]),
-          labels=np.array([0.0, 1.0]),
-          expected=0.6931472,
-      ),
+      {
+          'preds': np.array([-1e09, -1e-09]),
+          'labels': np.array([1.0, 0.0]),
+          'expected': 5e08,
+      },
+      {
+          'preds': np.array([-1e09, -1e-09]),
+          'labels': np.array([0.0, 1.0]),
+          'expected': 0.3465736,
+      },
+      {
+          'preds': np.array([1e09, 1e-09]),
+          'labels': np.array([1.0, 0.0]),
+          'expected': 0.3465736,
+      },
+      {
+          'preds': np.array([1e09, 1e-09]),
+          'labels': np.array([0.0, 1.0]),
+          'expected': 5e08,
+      },
+      {
+          'preds': np.array([-1e09, 1e-09]),
+          'labels': np.array([1.0, 0.0]),
+          'expected': 5e08,
+      },
+      {
+          'preds': np.array([-1e09, 1e-09]),
+          'labels': np.array([0.0, 1.0]),
+          'expected': 0.3465736,
+      },
+      {
+          'preds': np.array([1e09, -1e-09]),
+          'labels': np.array([1.0, 0.0]),
+          'expected': 0.3465736,
+      },
+      {
+          'preds': np.array([1e09, -1e-09]),
+          'labels': np.array([0.0, 1.0]),
+          'expected': 5e08,
+      },
+      {
+          'preds': np.array([0.0, 0.0]),
+          'labels': np.array([1.0, 0.0]),
+          'expected': 0.6931472,
+      },
+      {
+          'preds': np.array([0.0, 0.0]),
+          'labels': np.array([0.0, 1.0]),
+          'expected': 0.6931472,
+      },
   )
-  def testSigmoidCrossEntropy(self, preds, labels, expected):
+  def test_sigmoid_cross_entropy(self, preds, labels, expected):
     tested = jnp.mean(
         _classification.sigmoid_binary_cross_entropy(preds, labels)
     )
@@ -323,14 +370,14 @@ class PolyLossTest(parameterized.TestCase):
 
   @chex.all_variants
   @parameterized.parameters(
-      dict(eps=2, expected=4.5317),
-      dict(eps=1, expected=3.7153),
-      dict(eps=-1, expected=2.0827),
-      dict(eps=0, expected=2.8990),
-      dict(eps=-0.5, expected=2.4908),
-      dict(eps=1.15, expected=3.8378),
-      dict(eps=1.214, expected=3.8900),
-      dict(eps=5.45, expected=7.3480),
+      {'eps': 2, 'expected': 4.5317},
+      {'eps': 1, 'expected': 3.7153},
+      {'eps': -1, 'expected': 2.0827},
+      {'eps': 0, 'expected': 2.8990},
+      {'eps': -0.5, 'expected': 2.4908},
+      {'eps': 1.15, 'expected': 3.8378},
+      {'eps': 1.214, 'expected': 3.8900},
+      {'eps': 5.45, 'expected': 7.3480},
   )
   def test_scalar(self, eps, expected):
     np.testing.assert_allclose(
@@ -343,13 +390,13 @@ class PolyLossTest(parameterized.TestCase):
 
   @chex.all_variants
   @parameterized.parameters(
-      dict(eps=2, expected=np.array([0.4823, 1.2567])),
-      dict(eps=1, expected=np.array([0.3261, 1.0407])),
-      dict(eps=0, expected=np.array([0.1698, 0.8247])),
-      dict(eps=-0.5, expected=np.array([0.0917, 0.7168])),
-      dict(eps=1.15, expected=np.array([0.3495, 1.0731])),
-      dict(eps=1.214, expected=np.array([0.3595, 1.0870])),
-      dict(eps=5.45, expected=np.array([1.0211, 2.0018])),
+      {'eps': 2, 'expected': np.array([0.4823, 1.2567])},
+      {'eps': 1, 'expected': np.array([0.3261, 1.0407])},
+      {'eps': 0, 'expected': np.array([0.1698, 0.8247])},
+      {'eps': -0.5, 'expected': np.array([0.0917, 0.7168])},
+      {'eps': 1.15, 'expected': np.array([0.3495, 1.0731])},
+      {'eps': 1.214, 'expected': np.array([0.3595, 1.0870])},
+      {'eps': 5.45, 'expected': np.array([1.0211, 2.0018])},
   )
   def test_batched(self, eps, expected):
     np.testing.assert_allclose(
@@ -362,28 +409,28 @@ class PolyLossTest(parameterized.TestCase):
 
   @chex.all_variants
   @parameterized.parameters(
-      dict(
-          logits=np.array(
+      {
+          'logits': np.array(
               [[4.0, 2.0, 1.0], [0.0, 5.0, 1.0], [0.134, 1.234, 3.235]]
           ),
-          labels=np.array(
+          'labels': np.array(
               [[1.0, 0.0, 0.0], [0.0, 0.8, 0.2], [0.34, 0.33, 0.33]]
           ),
-      ),
-      dict(
-          logits=np.array([[4.0, 2.0, 1.0], [0.0, 5.0, 1.0]]),
-          labels=np.array([[1.0, 0.0, 0.0], [0.0, 0.8, 0.2]]),
-      ),
-      dict(
-          logits=np.array(
+      },
+      {
+          'logits': np.array([[4.0, 2.0, 1.0], [0.0, 5.0, 1.0]]),
+          'labels': np.array([[1.0, 0.0, 0.0], [0.0, 0.8, 0.2]]),
+      },
+      {
+          'logits': np.array(
               [[4.0, 2.0, 1.0, 0.134, 1.3515], [0.0, 5.0, 1.0, 0.5215, 5.616]]
           ),
-          labels=np.array(
+          'labels': np.array(
               [[0.5, 0.0, 0.0, 0.0, 0.5], [0.0, 0.12, 0.2, 0.56, 0.12]]
           ),
-      ),
-      dict(logits=np.array([1.89, 2.39]), labels=np.array([0.34, 0.66])),
-      dict(logits=np.array([0.314]), labels=np.array([1.0])),
+      },
+      {'logits': np.array([1.89, 2.39]), 'labels': np.array([0.34, 0.66])},
+      {'logits': np.array([0.314]), 'labels': np.array([1.0])},
   )
   def test_equals_to_cross_entropy_when_eps0(self, logits, labels):
     np.testing.assert_allclose(
@@ -394,7 +441,7 @@ class PolyLossTest(parameterized.TestCase):
         atol=1e-4,
     )
 
-  @parameterized.parameters(dict(size=5), dict(size=10))
+  @parameterized.parameters({'size': 5}, {'size': 10})
   def test_mask(self, size):
     preds = np.random.normal(size=size)
     targets = np.random.dirichlet(np.ones(size))
@@ -404,9 +451,9 @@ class PolyLossTest(parameterized.TestCase):
     np.testing.assert_allclose(x, y, atol=1e-4)
 
   @parameterized.parameters(
-      dict(axis=0, shape=[4, 5, 6]),
-      dict(axis=1, shape=[4, 5, 6]),
-      dict(axis=2, shape=[4, 5, 6]),
+      {'axis': 0, 'shape': [4, 5, 6]},
+      {'axis': 1, 'shape': [4, 5, 6]},
+      {'axis': 2, 'shape': [4, 5, 6]},
   )
   def test_axis(self, shape, axis):
     preds = np.random.normal(size=shape)
@@ -569,7 +616,7 @@ class ConvexKLDivergenceTest(parameterized.TestCase):
         atol=1e-4,
     )
 
-  @parameterized.parameters(dict(size=5), dict(size=10))
+  @parameterized.parameters({'size': 5}, {'size': 10})
   def test_mask(self, size):
     preds = np.random.normal(size=size)
     targets = np.random.dirichlet(np.ones(size))
@@ -579,9 +626,9 @@ class ConvexKLDivergenceTest(parameterized.TestCase):
     np.testing.assert_allclose(x, y, atol=1e-4)
 
   @parameterized.parameters(
-      dict(axis=0, shape=[4, 5, 6]),
-      dict(axis=1, shape=[4, 5, 6]),
-      dict(axis=2, shape=[4, 5, 6]),
+      {'axis': 0, 'shape': [4, 5, 6]},
+      {'axis': 1, 'shape': [4, 5, 6]},
+      {'axis': 2, 'shape': [4, 5, 6]},
   )
   def test_axis(self, shape, axis):
     preds = np.random.normal(size=shape)
@@ -601,7 +648,7 @@ class PerceptronTest(parameterized.TestCase):
     signed_label = jnp.array(2.0 * label - 1.0)
     score = jnp.array(10.0)
 
-    def reference_impl(label, logit) -> float:
+    def reference_impl(label, logit):
       return jax.nn.relu(-logit * (2.0 * label - 1.0))
 
     expected = reference_impl(label, score)
@@ -613,7 +660,7 @@ class PerceptronTest(parameterized.TestCase):
     signed_labels = jnp.array(2.0 * labels - 1.0)
     scores = jnp.array([10.0, 20.0])
 
-    def reference_impl(label, logit) -> float:
+    def reference_impl(label, logit):
       return jax.nn.relu(-logit * (2.0 * label - 1.0))
 
     expected = jax.vmap(reference_impl)(labels, scores)
@@ -675,7 +722,7 @@ class KLDivergenceTest(parameterized.TestCase):
         atol=1e-4,
     )
 
-  @parameterized.parameters(dict(size=5), dict(size=10))
+  @parameterized.parameters({'size': 5}, {'size': 10})
   def test_mask(self, size):
     preds = np.random.normal(size=size)
     targets = np.random.dirichlet(np.ones(size))
@@ -685,9 +732,9 @@ class KLDivergenceTest(parameterized.TestCase):
     np.testing.assert_allclose(x, y, atol=1e-4)
 
   @parameterized.parameters(
-      dict(axis=0, shape=[4, 5, 6]),
-      dict(axis=1, shape=[4, 5, 6]),
-      dict(axis=2, shape=[4, 5, 6]),
+      {'axis': 0, 'shape': [4, 5, 6]},
+      {'axis': 1, 'shape': [4, 5, 6]},
+      {'axis': 2, 'shape': [4, 5, 6]},
   )
   def test_axis(self, shape, axis):
     preds = np.random.normal(size=shape)
@@ -735,7 +782,7 @@ class KLDivergenceWithLogTargetsTest(parameterized.TestCase):
         atol=1e-4,
     )
 
-  @parameterized.parameters(dict(size=5), dict(size=10))
+  @parameterized.parameters({'size': 5}, {'size': 10})
   def test_mask(self, size):
     preds = np.random.normal(size=size)
     targets = np.log(np.random.dirichlet(np.ones(size)))
@@ -746,9 +793,9 @@ class KLDivergenceWithLogTargetsTest(parameterized.TestCase):
     np.testing.assert_allclose(x, y, atol=1e-4)
 
   @parameterized.parameters(
-      dict(axis=0, shape=[4, 5, 6]),
-      dict(axis=1, shape=[4, 5, 6]),
-      dict(axis=2, shape=[4, 5, 6]),
+      {'axis': 0, 'shape': [4, 5, 6]},
+      {'axis': 1, 'shape': [4, 5, 6]},
+      {'axis': 2, 'shape': [4, 5, 6]},
   )
   def test_axis(self, shape, axis):
     preds = np.random.normal(size=shape)
@@ -932,7 +979,11 @@ class SigmoidFocalLossTest(parameterized.TestCase):
     super().setUp()
     self.ys = np.array([[2.0, 0.1, -2.0], [0.3, -0.1, 1.2]], dtype=np.float32)
     self.ts = np.array([[0.0, 0.0, 1.0], [1.0, 0.0, 0.0]])
-    self._rtol = 5e-3 if jax.default_backend() != 'cpu' else 1e-6
+    # Relaxed tolerance to accommodate log-space numerical stability improvement
+    # The log-space focal loss implementation has slightly different numerical
+    # behavior for extreme values, which is expected and desirable for numerical
+    # stability
+    self._rtol = 5e-3 if jax.default_backend() != 'cpu' else 2e-5
 
     logit = lambda x: jnp.log(x / (1.0 - x))
     self.large_ys = logit(jnp.array([0.9, 0.98, 0.3, 0.99]))
@@ -950,6 +1001,27 @@ class SigmoidFocalLossTest(parameterized.TestCase):
         rtol=self._rtol,
     )
 
+  @chex.all_variants
+  def test_focal_gamma_zero_matches_binary_cross_entropy(self):
+    """sigmoid_focal_loss == sigmoid_binary_cross_entropy for gamma=0."""
+    # Test with various inputs to ensure consistency
+    test_logits = jnp.array([[-1.0, 0.0, 1.0], [2.0, -2.0, 0.5]])
+    test_labels = jnp.array([[0.0, 1.0, 1.0], [1.0, 0.0, 0.0]])
+
+    focal_loss = self.variant(_classification.sigmoid_focal_loss)(
+        test_logits, test_labels, gamma=0.0
+    )
+    binary_ce_loss = _classification.sigmoid_binary_cross_entropy(
+        test_logits, test_labels
+    )
+
+    np.testing.assert_allclose(
+        focal_loss,
+        binary_ce_loss,
+        rtol=self._rtol,
+        err_msg='Focal loss with gamma=0 should match binary cross-entropy'
+    )
+
   @chex.all_variants
   def test_scale(self):
     """This test should catch problems with p_t."""
@@ -1022,6 +1094,36 @@ class SigmoidFocalLossTest(parameterized.TestCase):
     assert all(ce_loss[self.ts == 0] > 0)
     assert all(focal_loss[self.ts == 0] == 0)
 
+  @chex.all_variants
+  def test_extreme_logits_finite_gradients(self):
+    """Test that extreme logits with gamma < 1 produce finite gradients."""
+    # Test cases with very extreme logits and non-integer labels
+    extreme_logits = jnp.array([100.0, -100.0, 75.0, -75.0, 50.0, -50.0])
+    # Include non-integer labels to test soft label stability
+    labels = jnp.array([0.9, 0.1, 0.8, 0.2, 1.0, 0.0])
+
+    # Test with gamma < 1 which is most problematic for numerical stability
+    gamma = 0.5
+    def loss_fn(logits):
+      return jnp.sum(self.variant(_classification.sigmoid_focal_loss)(
+          logits, labels, gamma=gamma
+      ))
+
+    # Compute loss and gradients
+    loss_value = loss_fn(extreme_logits)
+    gradients = jax.grad(loss_fn)(extreme_logits)
+
+    # Verify that both loss and gradients are finite
+    self.assertTrue(jnp.isfinite(loss_value),
+                    f'Loss should be finite for {gamma=}, got {loss_value}')
+    self.assertTrue(jnp.all(jnp.isfinite(gradients)),
+                    f'Gradients should be finite for {gamma=}, got {gradients}')
+
+    # Test Hessians for numerical stability
+    hessian = jax.hessian(loss_fn)(extreme_logits)
+    self.assertTrue(jnp.all(jnp.isfinite(hessian)),
+                    f'Hessians should be finite for {gamma=}')
+
 
 if __name__ == '__main__':
   absltest.main()
