@@ -35,11 +35,11 @@ def ntxent(
     >>> x = jax.random.normal(key1, shape=(4,2))
     >>> labels = jnp.array([0, 0, 1, 1])
     >>>
-    >>> print("input:", x)
-    input: [[-0.9155995   1.5534698 ]
-     [ 0.2623586  -1.5908985 ]
-     [-0.15977189  0.480501  ]
-     [ 0.58389133  0.10497775]]
+    >>> print("input:", x)  # doctest: +SKIP
+    input: [[ 0.07592554 -0.48634264]
+     [ 1.2903206   0.5196119 ]
+     [ 0.30040437  0.31034866]
+     [ 0.5761609  -0.8074621 ]]
     >>> print("labels:", labels)
     labels: [0 0 1 1]
     >>>
@@ -47,13 +47,13 @@ def ntxent(
     >>> b = jax.random.normal(key3, shape=(1,)) # params
     >>> out = x @ w + b # model
     >>>
-    >>> print("Embeddings:", out)
-    Embeddings: [[-1.0076267]
-     [-1.2960069]
-     [-1.1829865]
-     [-1.3485558]]
+    >>> print("Embeddings:", out)  # doctest: +SKIP
+    Embeddings: [[0.08969027]
+     [1.6291292 ]
+     [0.8622629 ]
+     [0.13612625]]
     >>> loss = optax.ntxent(out, labels)
-    >>> print("loss:", loss)
+    >>> print("loss:", loss)  # doctest: +SKIP
     loss: 1.0986123
 
   Args:
@@ -115,6 +115,60 @@ def ntxent(
   denom = jnp.sum(jnp.exp(xcs_shift_diffs), axis=1, keepdims=True)
   denom += numer_exp
   log_softm = numer - jnp.log(denom)
-  loss = -jnp.where(matches == 1, log_softm, 0.0).sum() / matches.sum()
+  loss = -jnp.where(matches == 1, log_softm, 0.0).sum()/matches.sum()
 
   return loss
+
+
+def triplet_margin_loss(
+    anchors: chex.Array,
+    positives: chex.Array,
+    negatives: chex.Array,
+    axis: int = -1,
+    norm_degree: chex.Numeric = 2,
+    margin: chex.Numeric = 1.0,
+    eps: chex.Numeric = 1e-6,
+) -> chex.Array:
+  """Returns the triplet loss for a batch of embeddings.
+
+  Examples:
+    >>> import jax.numpy as jnp, optax, chex
+    >>> jnp.set_printoptions(precision=4)
+    >>> anchors = jnp.array([[0.0, 0.0], [1.0, 1.0]])
+    >>> positives = jnp.array([[0.1, 0.1], [1.1, 1.1]])
+    >>> negatives = jnp.array([[1.0, 0.0], [0.0, 1.0]])
+    >>> output = optax.losses.triplet_margin_loss(anchors, positives, negatives,
+    ...                                           margin=1.0)
+    >>> print(output)
+    [0.1414 0.1414]
+
+  Args:
+    anchors: An array of anchor embeddings, with shape [batch, feature_dim].
+    positives: An array of positive embeddings (similar to anchors), with
+      shape [batch, feature_dim].
+    negatives: An array of negative embeddings (dissimilar to anchors), with
+      shape [batch, feature_dim].
+    axis: The axis along which to compute the distances (default is -1).
+    norm_degree: The norm degree for distance calculation (default is 2 for
+      Euclidean distance).
+    margin: The minimum margin by which the positive distance should be
+      smaller than the negative distance.
+    eps: A small epsilon value to ensure numerical stability in the distance
+      calculation.
+
+  Returns:
+    Returns the computed triplet loss as an array.
+
+  References:
+      V. Balntas et al,
+      `Learning shallow convolutional feature descriptors with triplet losses
+      <https://bmva-archive.org.uk/bmvc/2016/papers/paper119/abstract119.pdf>`
+      _, 2016
+  """
+  chex.assert_type([anchors, positives, negatives], float)
+  positive_distance = jnp.power(jnp.power(anchors - positives, norm_degree)
+                                .sum(axis) + eps, 1/norm_degree)
+  negative_distance = jnp.power(jnp.power(anchors - negatives, norm_degree)
+                                .sum(axis) + eps, 1/norm_degree)
+  loss = jnp.maximum(positive_distance - negative_distance + margin, 0)
+  return loss
