@@ -93,7 +93,7 @@ def control_delta_method(
     state.
 
   .. deprecated:: 0.2.4
-    This function will be removed in 0.3.0
+    This function will be removed in 0.2.7
   """
 
   def delta(
@@ -171,7 +171,7 @@ def moving_avg_baseline(
     state.
 
   .. deprecated:: 0.2.4
-    This function will be removed in 0.3.0
+    This function will be removed in 0.2.7
   """
 
   def moving_avg(
@@ -281,7 +281,7 @@ def control_variates_jacobians(
     * The updated CV state.
 
   .. deprecated:: 0.2.4
-    This function will be removed in 0.3.0
+    This function will be removed in 0.2.7
   """
   control_variate = control_variate_from_function(function)
   stochastic_cv, expected_value_cv, update_state_cv = control_variate
@@ -310,7 +310,7 @@ def control_variates_jacobians(
   # gradient.
   # The rng has to be the same as passed to the grad_estimator above so that we
   # obtain the same samples.
-  samples = dist_builder(*params).sample((num_samples,), seed=rng)
+  samples = dist_builder(*params).sample((num_samples,), key=rng)
   # If the CV has state, update it.
   control_variate_state = update_state_cv(
       params, samples, control_variate_state
@@ -418,7 +418,7 @@ def estimate_control_variate_coefficients(
     in `params`.
 
   .. deprecated:: 0.2.4
-    This function will be removed in 0.3.0
+    This function will be removed in 0.2.7
   """
   # Resample to avoid biased gradients.
   cv_rng, _ = jax.random.split(rng)
@@ -455,7 +455,7 @@ def estimate_control_variate_coefficients(
     cv_coeff = jnp.sum(cov) / (jnp.sum(jnp.var(param_cv_jacs, axis=0)) + eps)
     return jax.lax.stop_gradient(cv_coeff)
 
-  return [
+  return [  # pytype: disable=bad-return-type
       compute_coeff(cv_jacobians[i], function_jacobians[i])
       for i in range(len(params))
   ]
