@@ -17,6 +17,8 @@
 # pylint: disable=wrong-import-position
 # pylint: disable=g-importing-member
 
+import typing as _typing
+
 from optax import assignment
 from optax import contrib
 from optax import losses
@@ -26,6 +28,7 @@ from optax import projections
 from optax import schedules
 from optax import second_order
 from optax import transforms
+from optax import tree
 from optax import tree_utils
 from optax._src.alias import adabelief
 from optax._src.alias import adadelta
@@ -48,6 +51,7 @@ from optax._src.alias import nadamw
 from optax._src.alias import noisy_sgd
 from optax._src.alias import novograd
 from optax._src.alias import optimistic_adam
+from optax._src.alias import optimistic_adam_v2
 from optax._src.alias import optimistic_gradient_descent
 from optax._src.alias import polyak_sgd
 from optax._src.alias import radam
@@ -61,6 +65,7 @@ from optax._src.base import EmptyState
 from optax._src.base import GradientTransformation
 from optax._src.base import GradientTransformationExtraArgs
 from optax._src.base import identity
+from optax._src.base import init_empty_state
 from optax._src.base import OptState
 from optax._src.base import Params
 from optax._src.base import ScalarOrSchedule
@@ -73,27 +78,11 @@ from optax._src.base import TransformUpdateExtraArgsFn
 from optax._src.base import TransformUpdateFn
 from optax._src.base import Updates
 from optax._src.base import with_extra_args_support
-from optax._src.clipping import adaptive_grad_clip
-from optax._src.clipping import AdaptiveGradClipState
-from optax._src.clipping import clip
-from optax._src.clipping import clip_by_block_rms
-from optax._src.clipping import clip_by_global_norm
-from optax._src.clipping import ClipByGlobalNormState
-from optax._src.clipping import ClipState
-from optax._src.clipping import per_example_global_norm_clip
-from optax._src.clipping import per_example_layer_norm_clip
-from optax._src.combine import chain
-from optax._src.combine import multi_transform
-from optax._src.combine import MultiTransformState
-from optax._src.combine import named_chain
-from optax._src.constrain import keep_params_nonnegative
-from optax._src.constrain import NonNegativeParamsState
-from optax._src.constrain import zero_nans
-from optax._src.constrain import ZeroNansState
 from optax._src.factorized import FactoredState
 from optax._src.factorized import scale_by_factored_rms
 from optax._src.linear_algebra import global_norm
 from optax._src.linear_algebra import matrix_inverse_pth_root
+from optax._src.linear_algebra import nnls
 from optax._src.linear_algebra import power_iteration
 from optax._src.linesearch import scale_by_backtracking_linesearch
 from optax._src.linesearch import scale_by_zoom_linesearch
@@ -107,15 +96,9 @@ from optax._src.numerics import safe_increment
 from optax._src.numerics import safe_int32_increment
 from optax._src.numerics import safe_norm
 from optax._src.numerics import safe_root_mean_squares
-from optax._src.transform import add_decayed_weights
-from optax._src.transform import add_noise
-from optax._src.transform import AddDecayedWeightsState
-from optax._src.transform import AddNoiseState
 from optax._src.transform import apply_every
 from optax._src.transform import ApplyEvery
 from optax._src.transform import centralize
-from optax._src.transform import ema
-from optax._src.transform import EmaState
 from optax._src.transform import normalize_by_update_norm
 from optax._src.transform import scale
 from optax._src.transform import scale_by_adadelta
@@ -157,34 +140,71 @@ from optax._src.transform import ScaleByRssState
 from optax._src.transform import ScaleByRStdDevState
 from optax._src.transform import ScaleByScheduleState
 from optax._src.transform import ScaleBySM3State
-from optax._src.transform import ScaleByTrustRatioState
-from optax._src.transform import ScaleState
-from optax._src.transform import trace
-from optax._src.transform import TraceState
 from optax._src.update import apply_updates
 from optax._src.update import incremental_update
 from optax._src.update import periodic_update
 from optax._src.utils import multi_normal
 from optax._src.utils import scale_gradient
 from optax._src.utils import value_and_grad_from_state
-from optax._src.wrappers import apply_if_finite
-from optax._src.wrappers import ApplyIfFiniteState
-from optax._src.wrappers import conditionally_mask
-from optax._src.wrappers import conditionally_transform
-from optax._src.wrappers import ConditionallyMaskState
-from optax._src.wrappers import ConditionallyTransformState
-from optax._src.wrappers import flatten
-from optax._src.wrappers import masked
-from optax._src.wrappers import MaskedNode
-from optax._src.wrappers import MaskedState
-from optax._src.wrappers import maybe_update
-from optax._src.wrappers import MaybeUpdateState
-from optax._src.wrappers import MultiSteps
-from optax._src.wrappers import MultiStepsState
-from optax._src.wrappers import ShouldSkipUpdateFunction
-from optax._src.wrappers import skip_large_updates
-from optax._src.wrappers import skip_not_finite
 
+# TODO(mtthss): remove contrib aliases from flat namespace once users updated.
+# Deprecated modules
+from optax.contrib import (differentially_private_aggregate
+                           as _deprecated_differentially_private_aggregate)
+from optax.contrib import (DifferentiallyPrivateAggregateState
+                           as _deprecated_DifferentiallyPrivateAggregateState)
+from optax.contrib import dpsgd as _deprecated_dpsgd
+
+
+# TODO(mtthss): remove aliases after updates.
+adaptive_grad_clip = transforms.adaptive_grad_clip
+AdaptiveGradClipState = EmptyState
+clip = transforms.clip
+clip_by_block_rms = transforms.clip_by_block_rms
+clip_by_global_norm = transforms.clip_by_global_norm
+ClipByGlobalNormState = EmptyState
+ClipState = EmptyState
+per_example_global_norm_clip = transforms.per_example_global_norm_clip
+per_example_layer_norm_clip = transforms.per_example_layer_norm_clip
+keep_params_nonnegative = transforms.keep_params_nonnegative
+NonNegativeParamsState = transforms.NonNegativeParamsState
+zero_nans = transforms.zero_nans
+ZeroNansState = transforms.ZeroNansState
+chain = transforms.chain
+partition = transforms.partition
+PartitionState = transforms.PartitionState
+multi_transform = transforms.partition  # for backwards compatibility
+MultiTransformState = transforms.PartitionState  # for backwards compatibility
+named_chain = transforms.named_chain
+trace = transforms.trace
+TraceState = transforms.TraceState
+ema = transforms.ema
+EmaState = transforms.EmaState
+add_noise = transforms.add_noise
+AddNoiseState = transforms.AddNoiseState
+add_decayed_weights = transforms.add_decayed_weights
+AddDecayedWeightsState = EmptyState
+ScaleByTrustRatioState = EmptyState
+ScaleState = EmptyState
+apply_if_finite = transforms.apply_if_finite
+ApplyIfFiniteState = transforms.ApplyIfFiniteState
+conditionally_mask = transforms.conditionally_mask
+conditionally_transform = transforms.conditionally_transform
+ConditionallyMaskState = transforms.ConditionallyMaskState
+ConditionallyTransformState = transforms.ConditionallyTransformState
+flatten = transforms.flatten
+freeze = transforms.freeze
+selective_transform = transforms.selective_transform
+masked = transforms.masked
+MaskedNode = transforms.MaskedNode
+MaskedState = transforms.MaskedState
+MultiSteps = transforms.MultiSteps
+MultiStepsState = transforms.MultiStepsState
+ShouldSkipUpdateFunction = transforms.ShouldSkipUpdateFunction
+skip_large_updates = transforms.skip_large_updates
+skip_not_finite = transforms.skip_not_finite
+snapshot = transforms.snapshot
+SnapshotState = transforms.SnapshotState
 
 # TODO(mtthss): remove tree_utils aliases after updates.
 tree_map_params = tree_utils.tree_map_params
@@ -193,7 +213,7 @@ update_infinity_moment = tree_utils.tree_update_infinity_moment
 update_moment = tree_utils.tree_update_moment
 update_moment_per_elem_norm = tree_utils.tree_update_moment_per_elem_norm
 
-# TODO(mtthss): remove schedules alises from flat namespaces after user updates.
+# TODO(mtthss): remove schedules aliases from flat namespaces after user updates
 constant_schedule = schedules.constant_schedule
 cosine_decay_schedule = schedules.cosine_decay_schedule
 cosine_onecycle_schedule = schedules.cosine_onecycle_schedule
@@ -236,13 +256,6 @@ softmax_cross_entropy_with_integer_labels = (
 squared_error = losses.squared_error
 sigmoid_focal_loss = losses.sigmoid_focal_loss
 
-# pylint: disable=g-import-not-at-top
-# TODO(mtthss): remove contrib aliases from flat namespace once users updated.
-# Deprecated modules
-from optax.contrib import differentially_private_aggregate as _deprecated_differentially_private_aggregate
-from optax.contrib import DifferentiallyPrivateAggregateState as _deprecated_DifferentiallyPrivateAggregateState
-from optax.contrib import dpsgd as _deprecated_dpsgd
-
 _deprecations = {
     # Added Apr 2024
     "differentially_private_aggregate": (
@@ -269,9 +282,8 @@ _deprecations = {
         _deprecated_dpsgd,
     ),
 }
+# pylint: disable=g-import-not-at-top
 # pylint: disable=g-bad-import-order
-import typing as _typing
-
 if _typing.TYPE_CHECKING:
   # pylint: disable=reimported
   from optax.contrib import differentially_private_aggregate
@@ -280,7 +292,8 @@ if _typing.TYPE_CHECKING:
   # pylint: enable=reimported
 
 else:
-  from optax._src.deprecations import deprecation_getattr as _deprecation_getattr
+  from optax._src.deprecations import (deprecation_getattr
+                                       as _deprecation_getattr)
 
   __getattr__ = _deprecation_getattr(__name__, _deprecations)
   del _deprecation_getattr
@@ -290,7 +303,7 @@ del _typing
 # pylint: enable=g-importing-member
 
 
-__version__ = '0.2.4'
+__version__ = '0.2.6'
 
 __all__ = (
     "adabelief",
@@ -371,21 +384,22 @@ __all__ = (
     "MaskOrFn",
     "MaskedState",
     "matrix_inverse_pth_root",
-    "maybe_update",
-    "MaybeUpdateState",
     "multi_normal",
-    "multi_transform",
+    "multi_transform",  # for backwards compatibility
     "MultiSteps",
     "MultiStepsState",
-    "MultiTransformState",
+    "MultiTransformState",  # for backwards compatibility
     "nadam",
     "nadamw",
+    "nnls",
     "noisy_sgd",
     "novograd",
     "NonNegativeParamsState",
     "ntxent",
     "OptState",
     "Params",
+    "partition",
+    "PartitionState",
     "periodic_update",
     "per_example_global_norm_clip",
     "per_example_layer_norm_clip",
