@@ -28,37 +28,37 @@ from optax._src import update
 
 ALL_MODULES = [
     ('identity', base.identity, {}),
-    ('clip', clipping.clip, dict(max_delta=1.0)),
-    ('clip_by_global_norm', clipping.clip_by_global_norm, dict(max_norm=1.0)),
-    ('trace', transform.trace, dict(decay=0.5, nesterov=False)),
-    ('trace_with_nesterov', transform.trace, dict(decay=0.5, nesterov=True)),
+    ('clip', clipping.clip, {'max_delta': 1.0}),
+    ('clip_by_global_norm', clipping.clip_by_global_norm, {'max_norm': 1.0}),
+    ('trace', transform.trace, {'decay': 0.5, 'nesterov': False}),
+    ('trace_with_nesterov', transform.trace, {'decay': 0.5, 'nesterov': True}),
     ('scale_by_rss', transform.scale_by_rss, {}),
     ('scale_by_rms', transform.scale_by_rms, {}),
     ('scale_by_stddev', transform.scale_by_stddev, {}),
     ('adam', transform.scale_by_adam, {}),
-    ('scale', transform.scale, dict(step_size=3.0)),
+    ('scale', transform.scale, {'step_size': 3.0}),
     (
         'add_decayed_weights',
         transform.add_decayed_weights,
-        dict(weight_decay=0.1),
+        {'weight_decay': 0.1},
     ),
     (
         'scale_by_schedule',
         transform.scale_by_schedule,
-        dict(step_size_fn=lambda x: x * 0.1),
+        {'step_size_fn': lambda x: x * 0.1},
     ),
     ('scale_by_trust_ratio', transform.scale_by_trust_ratio, {}),
-    ('add_noise', transform.add_noise, dict(eta=1.0, gamma=0.1, seed=42)),
+    ('add_noise', transform.add_noise, {'eta': 1.0, 'gamma': 0.1, 'key': 0}),
     ('apply_every_k', transform.apply_every, {}),
-    ('adagrad', alias.adagrad, dict(learning_rate=0.1)),
-    ('adam', alias.adam, dict(learning_rate=0.1)),
-    ('adamw', alias.adamw, dict(learning_rate=0.1)),
-    ('fromage', alias.fromage, dict(learning_rate=0.1)),
-    ('lamb', alias.lamb, dict(learning_rate=0.1)),
-    ('noisy_sgd', alias.noisy_sgd, dict(learning_rate=0.1)),
-    ('rmsprop', alias.rmsprop, dict(learning_rate=0.1)),
-    ('sgd', alias.sgd, dict(learning_rate=0.1)),
-    ('sign_sgd', alias.sgd, dict(learning_rate=0.1)),
+    ('adagrad', alias.adagrad, {'learning_rate': 0.1}),
+    ('adam', alias.adam, {'learning_rate': 0.1}),
+    ('adamw', alias.adamw, {'learning_rate': 0.1}),
+    ('fromage', alias.fromage, {'learning_rate': 0.1}),
+    ('lamb', alias.lamb, {'learning_rate': 0.1}),
+    ('noisy_sgd', alias.noisy_sgd, {'learning_rate': 0.1, 'key': 0}),
+    ('rmsprop', alias.rmsprop, {'learning_rate': 0.1}),
+    ('sgd', alias.sgd, {'learning_rate': 0.1}),
+    ('sign_sgd', alias.sgd, {'learning_rate': 0.1}),
 ]
 
 
