@@ -34,7 +34,7 @@ from optax._src import combine
 from optax._src import linesearch as _linesearch
 from optax._src import update
 from optax._src import utils
-import optax.tree_utils as otu
+import optax.tree
 import scipy.optimize as scipy_optimize
 
 
@@ -70,16 +70,16 @@ def get_problem(name: str):
     sum2 = (0.5 * ii * x).sum()
     return sum1 + sum2**2 + sum2**4
 
-  problems = dict(
-      polynomial=dict(fn=polynomial, input_shape=()),
-      exponential=dict(fn=exponential, input_shape=()),
-      sinusoidal=dict(fn=sinusoidal, input_shape=()),
-      rosenbrock=dict(fn=rosenbrock, input_shape=(16,)),
-      himmelblau=dict(fn=himmelblau, input_shape=(2,)),
-      matyas=dict(fn=matyas, input_shape=(2,)),
-      eggholder=dict(fn=eggholder, input_shape=(2,)),
-      zakharov=dict(fn=zakharov, input_shape=(6,)),
-  )
+  problems = {
+      'polynomial': {'fn': polynomial, 'input_shape': ()},
+      'exponential': {'fn': exponential, 'input_shape': ()},
+      'sinusoidal': {'fn': sinusoidal, 'input_shape': ()},
+      'rosenbrock': {'fn': rosenbrock, 'input_shape': (16,)},
+      'himmelblau': {'fn': himmelblau, 'input_shape': (2,)},
+      'matyas': {'fn': matyas, 'input_shape': (2,)},
+      'eggholder': {'fn': eggholder, 'input_shape': (2,)},
+      'zakharov': {'fn': zakharov, 'input_shape': (6,)},
+  }
   return problems[name]
 
 
@@ -93,7 +93,7 @@ class BacktrackingLinesearchTest(chex.TestCase):
     final_value = fun(final_params)
     final_lr = final_state[0]
 
-    slope = otu.tree_vdot(descent_dir, init_grad)
+    slope = optax.tree.vdot(descent_dir, init_grad)
     slope_rtol, atol, rtol = (
         opt_args['slope_rtol'],
         opt_args['atol'],
@@ -117,7 +117,7 @@ class BacktrackingLinesearchTest(chex.TestCase):
     state = opt.init(params)
 
     # At initialization the value is inf
-    value = otu.tree_get(state, 'value')
+    value = optax.tree.get(state, 'value')
     self.assertTrue(jnp.isinf(value))
 
     update_fn = functools.partial(opt.update, value_fn=fun)
@@ -127,7 +127,7 @@ class BacktrackingLinesearchTest(chex.TestCase):
     )
 
     # If the step worked the value in the state should not be inf after one step
-    value = otu.tree_get(state, 'value')
+    value = optax.tree.get(state, 'value')
     self.assertFalse(jnp.isinf(value))
 
   @parameterized.product(
@@ -148,7 +148,7 @@ class BacktrackingLinesearchTest(chex.TestCase):
       seed,
   ):
     """Test backtracking linesearch (single update step)."""
-    key = jrd.PRNGKey(seed)
+    key = jrd.key(seed)
     problem = get_problem(problem_name)
     fn, input_shape = problem['fn'], problem['input_shape']
     init_params = jrd.normal(key, input_shape)
@@ -156,13 +156,13 @@ class BacktrackingLinesearchTest(chex.TestCase):
     base_opt = alias.sgd(learning_rate=1.0)
     descent_dir = -jax.grad(fn)(init_params)
 
-    opt_args = dict(
-        max_backtracking_steps=50,
-        slope_rtol=slope_rtol,
-        increase_factor=increase_factor,
-        atol=atol,
-        rtol=rtol,
-    )
+    opt_args = {
+        'max_backtracking_steps': 50,
+        'slope_rtol': slope_rtol,
+        'increase_factor': increase_factor,
+        'atol': atol,
+        'rtol': rtol,
+    }
 
     solver = combine.chain(
         base_opt,
@@ -270,7 +270,7 @@ class BacktrackingLinesearchTest(chex.TestCase):
 
     # Create artificial data
     noise = 1e-3
-    key = jrd.PRNGKey(0)
+    key = jrd.key(0)
     x_key, y_key, params_key = jrd.split(key, 3)
     d, m, n = 2, 16, 2
     xs = jrd.normal(x_key, (n, m, d))
@@ -305,6 +305,28 @@ class BacktrackingLinesearchTest(chex.TestCase):
         params, target_params, atol=5 * 1e-2, rtol=5 * 1e-2
     )
 
+  @parameterized.product(
+      dtype=(jnp.float16, jnp.bfloat16, jnp.float32, jnp.float64),
+      confuse_dtype=(jnp.float16, jnp.bfloat16, jnp.float32, jnp.float64),
+      kw=['slope_rtol', 'decrease_factor', 'increase_factor',
+          'max_learning_rate', 'atol', 'rtol']
+  )
+  def test_dtype_stability(self, dtype, confuse_dtype, kw):
+    with utils.x64_precision(True):
+      # pytype: disable=wrong-arg-types
+      opt = _linesearch.scale_by_backtracking_linesearch(
+          max_backtracking_steps=5,
+          **{kw: jnp.array(1e-5, dtype=confuse_dtype)})
+      # pytype: enable=wrong-arg-types
+      x = jnp.array([1.0, 2.0], dtype=dtype)
+      state = opt.init(x)
+      value_fn = lambda x: jnp.sum(x**2).astype(confuse_dtype)
+      # TODO(rdyro): ensure optimizer updates dtype matches parameters
+      cond = jax.random.randint(jax.random.key(0), (), 0, 2) == 0
+      jax.lax.cond(cond, lambda x: opt.update(x, state, x, value=1.0, grad=x,
+                                              value_fn=value_fn)[1],
+                   lambda x: state, x)
+
 
 def _run_linesearch(
     opt: base.GradientTransformationExtraArgs,
@@ -316,7 +338,7 @@ def _run_linesearch(
   """Runs the linesearch, i.e., a single update of scale_by_zoom_linesearch."""
   init_state = opt.init(params)
   if stepsize_guess is not None:
-    init_state = otu.tree_set(init_state, learning_rate=stepsize_guess)
+    init_state = optax.tree.set(init_state, learning_rate=stepsize_guess)
 
   value, grad = jax.value_and_grad(fn)(params)
   updates, final_state = opt.update(
@@ -341,8 +363,8 @@ class ZoomLinesearchTest(chex.TestCase):
   ) -> None:
     # Check that the value and gradient stored in the state
     # match the value and gradient of the step done with the stepsize found
-    final_value = otu.tree_get(final_state, 'value')
-    final_grad = otu.tree_get(final_state, 'grad')
+    final_value = optax.tree.get(final_state, 'value')
+    final_grad = optax.tree.get(final_state, 'grad')
     chex.assert_trees_all_close(
         value_fn(final_params), final_value, atol=1e-5, rtol=1e-5
     )
@@ -363,21 +385,21 @@ class ZoomLinesearchTest(chex.TestCase):
     """Check decrease conditions."""
     value_init, grad_init = jax.value_and_grad(fun)(init_params)
     value_final, grad_final = jax.value_and_grad(fun)(final_params)
-    final_lr = otu.tree_get(final_state, 'learning_rate')
-    num_linesearch_steps = otu.tree_get(final_state, 'num_linesearch_steps')
+    final_lr = optax.tree.get(final_state, 'learning_rate')
+    num_linesearch_steps = optax.tree.get(final_state, 'num_linesearch_steps')
     if allow_failure:
       potentially_failed = (
           num_linesearch_steps > opt_args['max_linesearch_steps']
       )
     else:
       potentially_failed = False
-    slope_init = otu.tree_vdot(updates, grad_init)
-    slope_final = otu.tree_vdot(updates, grad_final)
-    default_opt_args = dict(
-        slope_rtol=1e-4,
-        curv_rtol=0.9,
-        tol=0.0,
-    )
+    slope_init = optax.tree.vdot(updates, grad_init)
+    slope_final = optax.tree.vdot(updates, grad_final)
+    default_opt_args = {
+        'slope_rtol': 1e-4,
+        'curv_rtol': 0.9,
+        'tol': 0.0,
+    }
     opt_args = default_opt_args | opt_args
     slope_rtol, curv_rtol, tol = (
         opt_args['slope_rtol'],
@@ -414,7 +436,7 @@ class ZoomLinesearchTest(chex.TestCase):
     state = opt.init(params)
 
     # At initialization the value is inf
-    value = otu.tree_get(state, 'value')
+    value = optax.tree.get(state, 'value')
     self.assertTrue(jnp.isinf(value))
 
     update_fn = functools.partial(opt.update, value_fn=fun)
@@ -424,9 +446,10 @@ class ZoomLinesearchTest(chex.TestCase):
     )
 
     # If the step worked the value in the state should not be inf after one step
-    value = otu.tree_get(state, 'value')
+    value = optax.tree.get(state, 'value')
     self.assertFalse(jnp.isinf(value))
 
+  @absltest.skip('TODO(rdyro): need to match scipy linesearch algorithm')
   @parameterized.product(
       problem_name=[
           'polynomial',
@@ -446,7 +469,7 @@ class ZoomLinesearchTest(chex.TestCase):
     curv_rtol = 0.9
     tol = 0.0
 
-    key = jrd.PRNGKey(seed)
+    key = jrd.key(seed)
     params_key, precond_key = jrd.split(key, 2)
     problem = get_problem(problem_name)
     fn, input_shape = problem['fn'], problem['input_shape']
@@ -458,13 +481,13 @@ class ZoomLinesearchTest(chex.TestCase):
     # (non-negativity ensures that we keep a descent direction)
     init_updates = -precond_vec * jax.grad(fn)(init_params)
 
-    opt_args = dict(
-        max_linesearch_steps=30,
-        slope_rtol=slope_rtol,
-        curv_rtol=curv_rtol,
-        tol=tol,
-        max_learning_rate=None,
-    )
+    opt_args = {
+        'max_linesearch_steps': 30,
+        'slope_rtol': slope_rtol,
+        'curv_rtol': curv_rtol,
+        'tol': tol,
+        'max_learning_rate': None,
+    }
 
     opt = _linesearch.scale_by_zoom_linesearch(**opt_args)
     final_params, final_state = _run_linesearch(
@@ -483,8 +506,8 @@ class ZoomLinesearchTest(chex.TestCase):
           fn, init_params, init_updates, final_params, final_state, opt_args
       )
     with self.subTest('Check against scipy'):
-      stepsize = otu.tree_get(final_state, 'learning_rate')
-      final_value = otu.tree_get(final_state, 'value')
+      stepsize = optax.tree.get(final_state, 'learning_rate')
+      final_value = optax.tree.get(final_state, 'value')
       chex.assert_trees_all_close(scipy_res[0], stepsize, rtol=1e-5)
       chex.assert_trees_all_close(scipy_res[3], final_value, rtol=1e-5)
 
@@ -497,30 +520,30 @@ class ZoomLinesearchTest(chex.TestCase):
     # program
     if jax.default_backend() in ['tpu', 'gpu']:
       return
-    else:
-      # For this f and p, starting at a point on axis 0, the strong Wolfe
-      # condition 2 is met if and only if the step length s satisfies
-      # |x + s| <= c2 * |x|
-      def fn(w):
-        return jnp.dot(w, w)
 
-      u = jnp.array([1.0, 0.0])
-      w = 60 * u
+    # For this f and p, starting at a point on axis 0, the strong Wolfe
+    # condition 2 is met if and only if the step length s satisfies
+    # |x + s| <= c2 * |x|
+    def fn(w):
+      return jnp.dot(w, w)
 
-      # Test that the line search fails for p not a descent direction
-      # For high maxiter, still finds a decrease error because of
-      # the approximate Wolfe condition so we reduced maxiter
-      opt = _linesearch.scale_by_zoom_linesearch(
-          max_linesearch_steps=18, curv_rtol=0.5, verbose=True
-      )
-      stdout = io.StringIO()
-      with contextlib.redirect_stdout(stdout):
-        _, state = _run_linesearch(opt, fn, w, u, stepsize_guess=1.0)
-        stepsize = otu.tree_get(state, 'learning_rate')
-      # Check that we were not able to make a step or an infinitesimal one
-      self.assertLess(stepsize, 1e-5)
-      self.assertIn(_linesearch.FLAG_NOT_A_DESCENT_DIRECTION, stdout.getvalue())
-      self.assertIn(_linesearch.FLAG_NO_STEPSIZE_FOUND, stdout.getvalue())
+    u = jnp.array([1.0, 0.0])
+    w = 60 * u
+
+    # Test that the line search fails for p not a descent direction
+    # For high maxiter, still finds a decrease error because of
+    # the approximate Wolfe condition so we reduced maxiter
+    opt = _linesearch.scale_by_zoom_linesearch(
+        max_linesearch_steps=18, curv_rtol=0.5, verbose=True
+    )
+    stdout = io.StringIO()
+    with contextlib.redirect_stdout(stdout):
+      _, state = _run_linesearch(opt, fn, w, u, stepsize_guess=1.0)
+      stepsize = optax.tree.get(state, 'learning_rate')
+    # Check that we were not able to make a step or an infinitesimal one
+    self.assertLess(stepsize, 1e-5)
+    self.assertIn(_linesearch.FLAG_NOT_A_DESCENT_DIRECTION, stdout.getvalue())
+    self.assertIn(_linesearch.FLAG_NO_STEPSIZE_FOUND, stdout.getvalue())
 
   def test_failure_too_small_max_stepsize(self):
     """Check failure when the max stepsize is too small."""
@@ -531,32 +554,31 @@ class ZoomLinesearchTest(chex.TestCase):
     # program
     if jax.default_backend() in ['tpu', 'gpu']:
       return
-    else:
 
-      def fn(x):
-        return jnp.dot(x, x)
+    def fn(x):
+      return jnp.dot(x, x)
 
-      u = jnp.array([1.0, 0.0])
-      w = -60 * u
+    u = jnp.array([1.0, 0.0])
+    w = -60 * u
 
-      # Test that the line search fails if the maximum stepsize is too small
-      # Here, smallest s satisfying strong Wolfe conditions for c2=0.5 is 30
-      opt = _linesearch.scale_by_zoom_linesearch(
-          max_linesearch_steps=10,
-          curv_rtol=0.5,
-          verbose=True,
-          max_learning_rate=10.0,
-      )
-      stdout = io.StringIO()
-      with contextlib.redirect_stdout(stdout):
-        _, state = _run_linesearch(opt, fn, w, u, stepsize_guess=1.0)
-      stepsize = otu.tree_get(state, 'learning_rate')
-      # Check that we still made a step
-      self.assertEqual(stepsize, 10.0)
-      self.assertIn(_linesearch.FLAG_INTERVAL_NOT_FOUND, stdout.getvalue())
-      self.assertIn(
-          _linesearch.FLAG_CURVATURE_COND_NOT_SATISFIED, stdout.getvalue()
-      )
+    # Test that the line search fails if the maximum stepsize is too small
+    # Here, smallest s satisfying strong Wolfe conditions for c2=0.5 is 30
+    opt = _linesearch.scale_by_zoom_linesearch(
+        max_linesearch_steps=10,
+        curv_rtol=0.5,
+        verbose=True,
+        max_learning_rate=10.0,
+    )
+    stdout = io.StringIO()
+    with contextlib.redirect_stdout(stdout):
+      _, state = _run_linesearch(opt, fn, w, u, stepsize_guess=1.0)
+    stepsize = optax.tree.get(state, 'learning_rate')
+    # Check that we still made a step
+    self.assertEqual(stepsize, 10.0)
+    self.assertIn(_linesearch.FLAG_INTERVAL_NOT_FOUND, stdout.getvalue())
+    self.assertIn(
+        _linesearch.FLAG_CURVATURE_COND_NOT_SATISFIED, stdout.getvalue()
+    )
 
   def test_failure_not_enough_iter(self):
     """Check failure for a very small number of iterations."""
@@ -567,47 +589,46 @@ class ZoomLinesearchTest(chex.TestCase):
     # program
     if jax.default_backend() in ['tpu', 'gpu']:
       return
-    else:
 
-      def fn(x):
-        return jnp.dot(x, x)
+    def fn(x):
+      return jnp.dot(x, x)
 
-      u = jnp.array([1.0, 0.0])
-      w = -60 * u
+    u = jnp.array([1.0, 0.0])
+    w = -60 * u
 
-      curv_rtol = 0.5
-      # s=30 will only be tried on the 6th iteration, so this fails because
-      # the maximum number of iterations is reached.
-      opt = _linesearch.scale_by_zoom_linesearch(
-          max_linesearch_steps=5, curv_rtol=curv_rtol, verbose=True
-      )
-      stdout = io.StringIO()
-      with contextlib.redirect_stdout(stdout):
-        _, final_state = _run_linesearch(opt, fn, w, u, stepsize_guess=1.0)
-      stepsize = otu.tree_get(final_state, 'learning_rate')
-      # Check that we still made a step
-      self.assertEqual(stepsize, 16.0)
-      decrease_error = otu.tree_get(final_state, 'decrease_error')
-      curvature_error = otu.tree_get(final_state, 'curvature_error')
-      success = (decrease_error <= 0.0) and (curvature_error <= 0.0)
-      self.assertFalse(success)
-      # Here the error should not be that we haven't had a descent direction
-      self.assertNotIn(
-          _linesearch.FLAG_NOT_A_DESCENT_DIRECTION, stdout.getvalue()
-      )
+    curv_rtol = 0.5
+    # s=30 will only be tried on the 6th iteration, so this fails because
+    # the maximum number of iterations is reached.
+    opt = _linesearch.scale_by_zoom_linesearch(
+        max_linesearch_steps=5, curv_rtol=curv_rtol, verbose=True
+    )
+    stdout = io.StringIO()
+    with contextlib.redirect_stdout(stdout):
+      _, final_state = _run_linesearch(opt, fn, w, u, stepsize_guess=1.0)
+    stepsize = optax.tree.get(final_state, 'learning_rate')
+    # Check that we still made a step
+    self.assertEqual(stepsize, 16.0)
+    decrease_error = optax.tree.get(final_state, 'decrease_error')
+    curvature_error = optax.tree.get(final_state, 'curvature_error')
+    success = (decrease_error <= 0.0) and (curvature_error <= 0.0)
+    self.assertFalse(success)
+    # Here the error should not be that we haven't had a descent direction
+    self.assertNotIn(
+        _linesearch.FLAG_NOT_A_DESCENT_DIRECTION, stdout.getvalue()
+    )
 
-      # Check if it works normally
-      opt = _linesearch.scale_by_zoom_linesearch(
-          max_linesearch_steps=30, curv_rtol=curv_rtol
-      )
-      final_params, final_state = _run_linesearch(
-          opt, fn, w, u, stepsize_guess=1.0
-      )
-      s = otu.tree_get(final_state, 'learning_rate')
-      self._check_linesearch_conditions(
-          fn, w, u, final_params, final_state, dict(curv_rtol=curv_rtol)
-      )
-      self.assertGreaterEqual(s, 30.0)
+    # Check if it works normally
+    opt = _linesearch.scale_by_zoom_linesearch(
+        max_linesearch_steps=30, curv_rtol=curv_rtol
+    )
+    final_params, final_state = _run_linesearch(
+        opt, fn, w, u, stepsize_guess=1.0
+    )
+    s = optax.tree.get(final_state, 'learning_rate')
+    self._check_linesearch_conditions(
+        fn, w, u, final_params, final_state, {'curv_rtol': curv_rtol}
+    )
+    self.assertGreaterEqual(s, 30.0)
 
   def test_failure_flat_fun(self):
     """Check failure for a very flat function."""
@@ -618,20 +639,19 @@ class ZoomLinesearchTest(chex.TestCase):
     # program
     if jax.default_backend() in ['tpu', 'gpu']:
       return
-    else:
 
-      def fun_flat(x):
-        return jnp.exp(-1 / x**2)
+    def fun_flat(x):
+      return jnp.exp(-1 / x**2)
 
-      w = jnp.asarray(-0.2)
-      u = -jax.grad(fun_flat)(w)
-      opt = _linesearch.scale_by_zoom_linesearch(
-          max_linesearch_steps=30, verbose=True
-      )
-      stdout = io.StringIO()
-      with contextlib.redirect_stdout(stdout):
-        _, _ = _run_linesearch(opt, fun_flat, w, u, stepsize_guess=1.0)
-      self.assertIn(_linesearch.FLAG_INTERVAL_TOO_SMALL, stdout.getvalue())
+    w = jnp.asarray(-0.2)
+    u = -jax.grad(fun_flat)(w)
+    opt = _linesearch.scale_by_zoom_linesearch(
+        max_linesearch_steps=30, verbose=True
+    )
+    stdout = io.StringIO()
+    with contextlib.redirect_stdout(stdout):
+      _, _ = _run_linesearch(opt, fun_flat, w, u, stepsize_guess=1.0)
+    self.assertIn(_linesearch.FLAG_INTERVAL_TOO_SMALL, stdout.getvalue())
 
   def test_failure_inf_value(self):
     """Check behavior for inf/nan values."""
@@ -642,19 +662,18 @@ class ZoomLinesearchTest(chex.TestCase):
     # program
     if jax.default_backend() in ['tpu', 'gpu']:
       return
-    else:
 
-      def fun_inf(x):
-        return jnp.log(x)
+    def fun_inf(x):
+      return jnp.log(x)
 
-      w = jnp.asarray(1.0)
-      u = jnp.asarray(-2.0)
-      opt = _linesearch.scale_by_zoom_linesearch(
-          max_linesearch_steps=30, verbose=True
-      )
-      _, state = _run_linesearch(opt, fun_inf, w, u, stepsize_guess=1.0)
-      stepsize = otu.tree_get(state, 'learning_rate')
-      self.assertGreater(stepsize, 0.0)
+    w = jnp.asarray(1.0)
+    u = jnp.asarray(-2.0)
+    opt = _linesearch.scale_by_zoom_linesearch(
+        max_linesearch_steps=30, verbose=True
+    )
+    _, state = _run_linesearch(opt, fun_inf, w, u, stepsize_guess=1.0)
+    stepsize = optax.tree.get(state, 'learning_rate')
+    self.assertGreater(stepsize, 0.0)
 
   def test_high_smaller_than_low(self):
     # See google/jax/issues/16236
@@ -674,11 +693,33 @@ class ZoomLinesearchTest(chex.TestCase):
 
     opt = _linesearch.scale_by_zoom_linesearch(max_linesearch_steps=20)
     _, final_state = _run_linesearch(opt, fn, w, u, stepsize_guess=1.0)
-    decrease_error = otu.tree_get(final_state, 'decrease_error')
-    curvature_error = otu.tree_get(final_state, 'curvature_error')
+    decrease_error = optax.tree.get(final_state, 'decrease_error')
+    curvature_error = optax.tree.get(final_state, 'curvature_error')
     success = (decrease_error <= 0.0) and (curvature_error <= 0.0)
     self.assertTrue(success)
 
+  @parameterized.product(
+      dtype=(jnp.float16, jnp.bfloat16, jnp.float32, jnp.float64),
+      confuse_dtype=(jnp.float16, jnp.bfloat16, jnp.float32, jnp.float64),
+      kw=['tol', 'increase_factor', 'slope_rtol', 'curv_rtol',
+          'approx_dec_rtol', 'stepsize_precision']
+  )
+  def test_dtype_stability(self, dtype, confuse_dtype, kw):
+    with utils.x64_precision(True):
+      # pytype: disable=wrong-arg-types
+      opt = _linesearch.scale_by_zoom_linesearch(
+          max_linesearch_steps=5,
+          **{kw: jnp.array(1e-5, dtype=confuse_dtype)})
+      # pytype: enable=wrong-arg-types
+      x = jnp.array([1.0, 2.0], dtype=dtype)
+      state = opt.init(x)
+      value_fn = lambda x: jnp.sum(x**2).astype(confuse_dtype)
+      # TODO(rdyro): ensure optimizer updates dtype matches parameters
+      cond = jax.random.randint(jax.random.key(0), (), 0, 2) == 0
+      jax.lax.cond(cond, lambda x: opt.update(x, state, x, value=1.0, grad=x,
+                                              value_fn=value_fn)[1],
+                   lambda x: state, x)
+
 
 if __name__ == '__main__':
   absltest.main()
