@@ -28,8 +28,13 @@ from optax._src import numerics
 
 _ALL_ORDS = [None, np.inf, -np.inf, "fro", "nuc", 0, 1, 2, -2, -2, -1.5, 1.5]
 
-int32_array = lambda i: jnp.array(i, dtype=jnp.int32)
-float32_array = lambda i: jnp.array(i, dtype=jnp.float32)
+
+def int32_array(i):
+  return jnp.array(i, dtype=jnp.int32)
+
+
+def float32_array(i):
+  return jnp.array(i, dtype=jnp.float32)
 
 
 def _invalid_ord_axis_inputs(ord_axis_keepdims):
@@ -130,33 +135,39 @@ class NumericsTest(chex.TestCase):
     g = drms_dx(float32_array(0.0), float32_array(3.0))
     np.testing.assert_array_equal(g, jnp.zeros_like(g))
 
-  def test_complex_vs_real_abs_sqr(self):
+  @parameterized.product(
+      sq_fn=[lambda x: x**2, lambda x: x * x, jnp.square],
+      x=[
+          3,
+          3.0,
+          np.array([4, 5.2]),
+          1j,
+          3.0 + 1j,
+          np.array([4 + 1j, 5.2 + 1j]),
+      ],
+  )
+  def test_abs_sqr_hlo_equivalence(self, sq_fn, x):
     # Tests that JAX generates the same HLO from `numerics.abs_sq`,
-    # `jnp.square(x)`, `x * x`,  and `x**2`.
-    real_sq_fns = (lambda x: x**2, lambda x: x * x, jnp.square)
+    # `jnp.square(x)`, `x * x`,  and `x**2` if they are real
+    # Check that it does not generate the same hlo if they are complex.
 
     def _get_hlo_repr(f, x):
       hlo_string = jax.jit(f).lower(x).compiler_ir(dialect="hlo").as_hlo_text()
-      return re.sub(
-          "HloModule.*?\n", "", re.sub("ENTRY.*?{", "ENTRY XXXX", hlo_string)
+      hlo_string = re.sub("ENTRY.*?{", "ENTRY XXXX", hlo_string)
+      hlo_string = re.sub("HloModule.*?\n", "", hlo_string)
+      hlo_string = re.sub("ROOT.*?=", "ROOT result.2 =", hlo_string)
+      return hlo_string
+
+    if jnp.issubdtype(jnp.asarray(x).dtype, jnp.complexfloating):
+      self.assertNotEqual(
+          _get_hlo_repr(sq_fn, x),
+          _get_hlo_repr(numerics.abs_sq, x),
+      )
+    else:
+      self.assertEqual(
+          _get_hlo_repr(sq_fn, x),
+          _get_hlo_repr(numerics.abs_sq, x),
       )
-
-    # Real arg (same HLO).
-    for real_sq_fn in real_sq_fns:
-      for real_x in (3, 3.0, np.array([4, 5.2])):
-        self.assertEqual(
-            _get_hlo_repr(real_sq_fn, real_x),
-            _get_hlo_repr(numerics.abs_sq, real_x),
-        )
-
-    # Complex arg (different HLOs).
-    for real_sq_fn in real_sq_fns:
-      for complex_x in (1j, 3.0 + 1j, np.array([4 + 1j, 5.2 + 1j])):
-        self.assertNotEqual(
-            _get_hlo_repr(real_sq_fn, complex_x),
-            _get_hlo_repr(numerics.abs_sq, complex_x),
-        )
-
 
 if __name__ == "__main__":
   absltest.main()
