@@ -27,32 +27,43 @@ import jax
 import jax.numpy as jnp
 from optax import contrib
 from optax._src import alias
+from optax._src import base
 from optax._src import combine
 from optax._src import numerics
 from optax._src import update
+from optax._src import utils
 from optax.schedules import _inject
+from optax.schedules import _schedule
 from optax.transforms import _accumulation
-from optax.tree_utils import _state_utils
+import optax.tree
 
 # Testing contributions coded as GradientTransformations
 _MAIN_OPTIMIZERS_UNDER_TEST = [
-    dict(opt_name='acprop', opt_kwargs=dict(learning_rate=1e-3)),
-    dict(opt_name='cocob', opt_kwargs={}),
-    dict(opt_name='cocob', opt_kwargs=dict(weight_decay=1e-2)),
-    dict(opt_name='dadapt_adamw', opt_kwargs=dict(learning_rate=1e-1)),
-    dict(opt_name='dog', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='dowg', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='momo', opt_kwargs=dict(learning_rate=1e-1)),
-    dict(opt_name='momo_adam', opt_kwargs=dict(learning_rate=1e-1)),
-    dict(opt_name='prodigy', opt_kwargs=dict(learning_rate=1e-1)),
-    dict(
-        opt_name='schedule_free_sgd',
-        opt_kwargs=dict(learning_rate=1e-2, warmup_steps=5000),
-    ),
-    dict(
-        opt_name='schedule_free_adamw',
-        opt_kwargs=dict(learning_rate=1e-2, warmup_steps=5000),
-    ),
+    {'opt_name': 'acprop', 'opt_kwargs': {'learning_rate': 1e-3}},
+    {'opt_name': 'ademamix', 'opt_kwargs': {'learning_rate': 1e-3}},
+    {'opt_name': 'simplified_ademamix', 'opt_kwargs': {'learning_rate': 1e-3}},
+    {'opt_name': 'adopt', 'opt_kwargs': {'learning_rate': 1e-2}},
+    {'opt_name': 'cocob', 'opt_kwargs': {}},
+    {'opt_name': 'cocob', 'opt_kwargs': {'weight_decay': 1e-2}},
+    {'opt_name': 'dadapt_adamw', 'opt_kwargs': {'learning_rate': 1e-1}},
+    {'opt_name': 'dog', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'dowg', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'momo', 'opt_kwargs': {'learning_rate': 1e-1}},
+    {'opt_name': 'momo_adam', 'opt_kwargs': {'learning_rate': 1e-1}},
+    {'opt_name': 'muon', 'opt_kwargs': {'learning_rate': 1e-2}},
+    {'opt_name': 'prodigy', 'opt_kwargs': {'learning_rate': 1e-1}},
+    {
+        'opt_name': 'schedule_free_sgd',
+        'opt_kwargs': {'learning_rate': 1e-2, 'warmup_steps': 5000},
+    },
+    {
+        'opt_name': 'schedule_free_adamw',
+        'opt_kwargs': {'learning_rate': 1e-2, 'warmup_steps': 5000},
+    },
+    {
+        'opt_name': 'sophia',
+        'opt_kwargs': {'learning_rate': 1e-2}
+    },
 ]
 for optimizer in _MAIN_OPTIMIZERS_UNDER_TEST:
   optimizer['wrapper_name'] = None
@@ -61,59 +72,65 @@ for optimizer in _MAIN_OPTIMIZERS_UNDER_TEST:
 # Testing contributions coded as wrappers
 # (just with sgd as we just want the behavior of the wrapper)
 _MAIN_OPTIMIZERS_UNDER_TEST += [
-    dict(
-        opt_name='sgd',
-        opt_kwargs=dict(learning_rate=1e-1),
-        wrapper_name='mechanize',
-        wrapper_kwargs=dict(weight_decay=0.0),
-    ),
-    dict(
-        opt_name='sgd',
-        opt_kwargs=dict(learning_rate=1e-2),
-        wrapper_name='schedule_free',
-        wrapper_kwargs=dict(learning_rate=1e-2),
-    ),
-    dict(
-        opt_name='sgd',
-        opt_kwargs=dict(learning_rate=1e-3),
-        wrapper_name='reduce_on_plateau',
-        wrapper_kwargs={},
-    ),
+    {
+        'opt_name': 'sgd',
+        'opt_kwargs': {'learning_rate': 1e-1},
+        'wrapper_name': 'mechanize',
+        'wrapper_kwargs': {'weight_decay': 0.0},
+    },
+    {
+        'opt_name': 'sgd',
+        'opt_kwargs': {'learning_rate': 1e-2},
+        'wrapper_name': 'schedule_free',
+        'wrapper_kwargs': {'learning_rate': 1e-2},
+    },
+    {
+        'opt_name': 'sgd',
+        'opt_kwargs': {'learning_rate': 1e-3},
+        'wrapper_name': 'reduce_on_plateau',
+        'wrapper_kwargs': {},
+    },
 ]
 
 # Adding here instantiations of wrappers with any base optimizer
 _BASE_OPTIMIZERS = [
-    dict(opt_name='sgd', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='sgd', opt_kwargs=dict(learning_rate=1.0, momentum=0.9)),
-    dict(opt_name='adam', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='adamw', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='adamax', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='adamaxw', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='adan', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='amsgrad', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='lamb', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='lion', opt_kwargs=dict(learning_rate=1.0, b1=0.99)),
-    dict(opt_name='noisy_sgd', opt_kwargs=dict(learning_rate=1.0, eta=1e-4)),
-    dict(opt_name='novograd', opt_kwargs=dict(learning_rate=1.0)),
-    dict(
-        opt_name='optimistic_gradient_descent',
-        opt_kwargs=dict(learning_rate=1.0, alpha=0.7, beta=0.1),
-    ),
-    dict(opt_name='rmsprop', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='rmsprop', opt_kwargs=dict(learning_rate=1.0, momentum=0.9)),
-    dict(opt_name='adabelief', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='radam', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='sm3', opt_kwargs=dict(learning_rate=1.0)),
-    dict(opt_name='yogi', opt_kwargs=dict(learning_rate=1.0, b1=0.99)),
+    {'opt_name': 'sgd', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'sgd', 'opt_kwargs': {'learning_rate': 1.0, 'momentum': 0.9}},
+    {'opt_name': 'adam', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'adamw', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'adamax', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'adamaxw', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'adan', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'amsgrad', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'lamb', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'lion', 'opt_kwargs': {'learning_rate': 1.0, 'b1': 0.99}},
+    {
+        'opt_name': 'noisy_sgd',
+        'opt_kwargs': {'learning_rate': 1.0, 'eta': 1e-4, 'key': 0},
+    },
+    {'opt_name': 'novograd', 'opt_kwargs': {'learning_rate': 1.0}},
+    {
+        'opt_name': 'optimistic_gradient_descent',
+        'opt_kwargs': {'learning_rate': 1.0, 'alpha': 0.7, 'beta': 0.1},
+    },
+    {'opt_name': 'rmsprop', 'opt_kwargs': {'learning_rate': 1.0}},
+    {
+        'opt_name': 'rmsprop',
+        'opt_kwargs': {'learning_rate': 1.0, 'momentum': 0.9},
+    },
+    {'opt_name': 'adabelief', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'radam', 'opt_kwargs': {'learning_rate': 1.0}},
+    {'opt_name': 'sm3', 'opt_kwargs': {'learning_rate': 3.0}},
+    {'opt_name': 'yogi', 'opt_kwargs': {'learning_rate': 1.0, 'b1': 0.99}},
 ]
 # TODO(harshm): make LARS and Fromage work with mechanic.
 _OTHER_OPTIMIZERS_UNDER_TEST = [
-    dict(
-        opt_name=base_opt['opt_name'],
-        opt_kwargs=base_opt['opt_kwargs'],
-        wrapper_name='mechanize',
-        wrapper_kwargs=dict(weight_decay=0.0),
-    )
+    {
+        'opt_name': base_opt['opt_name'],
+        'opt_kwargs': base_opt['opt_kwargs'],
+        'wrapper_name': 'mechanize',
+        'wrapper_kwargs': {'weight_decay': 0.0},
+    }
     for base_opt in _BASE_OPTIMIZERS
 ]
 
@@ -135,8 +152,7 @@ def _get_opt_factory(opt_name):
 def _wrap_opt(opt, wrapper_name, wrapper_kwargs):
   if wrapper_name == 'reduce_on_plateau':
     return combine.chain(opt, contrib.reduce_on_plateau(**wrapper_kwargs))
-  else:
-    return getattr(contrib, wrapper_name)(opt, **wrapper_kwargs)
+  return getattr(contrib, wrapper_name)(opt, **wrapper_kwargs)
 
 
 def _setup_parabola(dtype):
@@ -144,11 +160,10 @@ def _setup_parabola(dtype):
   initial_params = jnp.array([-1.0, 10.0, 1.0], dtype=dtype)
   final_params = jnp.array([1.0, -1.0, 1.0], dtype=dtype)
 
-  @jax.value_and_grad
-  def get_updates(params):
+  def obj_fn(params):
     return jnp.sum(numerics.abs_sq(params - final_params))
 
-  return initial_params, final_params, get_updates
+  return initial_params, final_params, obj_fn
 
 
 def _setup_rosenbrock(dtype):
@@ -159,20 +174,85 @@ def _setup_rosenbrock(dtype):
   initial_params = jnp.array([0.0, 0.0], dtype=dtype)
   final_params = jnp.array([a, a**2], dtype=dtype)
 
-  @jax.value_and_grad
-  def get_updates(params):
+  def obj_fn(params):
     return numerics.abs_sq(a - params[0]) + b * numerics.abs_sq(
         params[1] - params[0] ** 2
     )
 
-  return initial_params, final_params, get_updates
+  return initial_params, final_params, obj_fn
+
+
+def _setup_matrix_parabola(dtype):
+  """Quadratic function as an optimization target with 2D tensor parameters."""
+  initial_params = jnp.zeros((2, 2), dtype=dtype)
+  final_params = jnp.array([[3.0, -2.0], [1.0, 4.0]], dtype=dtype)
+
+  def obj_fn(params):
+    return jnp.sum(numerics.abs_sq(params - final_params))
+
+  return initial_params, final_params, obj_fn
+
+
+def _setup_mixed_tensor_target(dtype):
+  """Optimization target combining 1D and 2D tensor parameters."""
+  initial_1d_params = jnp.zeros((3,), dtype=dtype)
+  final_1d_params = jnp.array([1.0, -1.0, 2.0], dtype=dtype)
+
+  initial_2d_params = jnp.zeros((2, 2), dtype=dtype)
+  final_2d_params = jnp.array([[1.0, 0.0], [-1.0, 1.0]], dtype=dtype)
+
+  def obj_fn(params):
+    """Objective function combining 1D and 2D parameters."""
+    params_1d, params_2d = params
+    loss_1d = jnp.sum(numerics.abs_sq(params_1d - final_1d_params))
+    loss_2d = jnp.sum(numerics.abs_sq(params_2d - final_2d_params))
+    return loss_1d + loss_2d
+
+  initial_params = (initial_1d_params, initial_2d_params)
+  final_params = (final_1d_params, final_2d_params)
+
+  return initial_params, final_params, obj_fn
 
 
 class ContribTest(chex.TestCase):
 
+  @parameterized.product(_ALL_OPTIMIZERS_UNDER_TEST, wrap=[True, False])
+  def test_optimizers_accept_extra_args(
+      self, opt_name, opt_kwargs, wrapper_name, wrapper_kwargs, wrap):
+    opt = _get_opt_factory(opt_name)(**opt_kwargs)
+    if wrap and wrapper_name is not None:
+      opt = _wrap_opt(opt, wrapper_name, wrapper_kwargs)
+    # intentionally ommit: opt = base.with_extra_args_support(opt)
+
+    initial_params, _, objective = _setup_rosenbrock(jnp.float32)
+
+    @jax.jit
+    def step(params, state):
+      value, updates = jax.value_and_grad(objective)(params)
+      update_kwargs = {'unexpected_extra_args_your_optimizer_doesnt_expect': 1}
+      if opt_name in ['momo', 'momo_adam', 'sgd']:
+        update_kwargs['value'] = value
+      if opt_name in ['sophia']:
+        update_kwargs['obj_fn'] = objective
+      updates, state = opt.update(updates, state, params, **update_kwargs)
+      params = update.apply_updates(params, updates)
+      return params, state
+
+    params = initial_params
+    state = opt.init(params)
+
+    with self.subTest('Test that update works with extra args'):
+      for _ in range(2):
+        params, state = step(params, state)
+
   @parameterized.product(
       _ALL_OPTIMIZERS_UNDER_TEST,
-      target=(_setup_parabola, _setup_rosenbrock),
+      target=(
+          _setup_parabola,
+          _setup_rosenbrock,
+          _setup_matrix_parabola,
+          _setup_mixed_tensor_target,
+      ),
       dtype=('float32',),
   )
   def test_optimizers(
@@ -188,16 +268,18 @@ class ContribTest(chex.TestCase):
     opt = _get_opt_factory(opt_name)(**opt_kwargs)
     if wrapper_name is not None:
       opt = _wrap_opt(opt, wrapper_name, wrapper_kwargs)
-    initial_params, final_params, get_updates = target(dtype)
+    initial_params, final_params, obj_fn = target(dtype)
 
     @jax.jit
     def step(params, state):
-      value, updates = get_updates(params)
+      value, updates = jax.value_and_grad(obj_fn)(params)
       if (
           opt_name in ['momo', 'momo_adam']
           or wrapper_name == 'reduce_on_plateau'
       ):
         update_kwargs = {'value': value}
+      elif opt_name == 'sophia':
+        update_kwargs = {'obj_fn': obj_fn}
       else:
         update_kwargs = {}
       updates, state = opt.update(updates, state, params, **update_kwargs)
@@ -208,7 +290,7 @@ class ContribTest(chex.TestCase):
     state = opt.init(params)
     with self.subTest('Test that tree_map_params works'):
       # A no-op change, to verify that tree map works.
-      state = _state_utils.tree_map_params(opt, lambda v: v, state)
+      state = optax.tree.map_params(opt, lambda v: v, state)
 
     with self.subTest('Test that optimization works'):
 
@@ -253,7 +335,8 @@ class ContribTest(chex.TestCase):
     # Add here the hyperparameters that cannot be injected with
     # inject_hyperparams.
     static_args = []
-    for uninjectable_hparam in ['warmup_steps', 'num_betas']:
+    for uninjectable_hparam in ['warmup_steps', 'num_betas', 'clip_value_fn',
+                                'ns_steps']:
       if uninjectable_hparam in inspect.signature(factory).parameters.keys():
         static_args.append(uninjectable_hparam)
     static_args = tuple(static_args)
@@ -266,14 +349,21 @@ class ContribTest(chex.TestCase):
       update_kwargs = {'value': jnp.array(1.0)}
     else:
       update_kwargs = {}
+    if opt_name == 'sophia':
+      obj_fn = lambda x: optax.tree.norm(x, squared=True)
+      update_fn = functools.partial(opt.update, obj_fn=obj_fn)
+      inject_update_fn = functools.partial(opt_inject.update, obj_fn=obj_fn)
+    else:
+      update_fn = opt.update
+      inject_update_fn = opt_inject.update
 
     state = self.variant(opt.init)(params)
-    updates, new_state = self.variant(opt.update)(
+    updates, new_state = self.variant(update_fn)(
         grads, state, params, **update_kwargs
     )
 
     state_inject = self.variant(opt_inject.init)(params)
-    updates_inject, new_state_inject = self.variant(opt_inject.update)(
+    updates_inject, new_state_inject = self.variant(inject_update_fn)(
         grads, state_inject, params, **update_kwargs
     )
 
@@ -299,12 +389,12 @@ class ContribTest(chex.TestCase):
     """Test that the optimizers return updates of same dtype as params."""
     # When debugging this test, note that operations like
     # x = 0.5**jnp.asarray(1, dtype=jnp.int32)
-    # (appearing in e.g. optax.tree_utils.tree_bias_correction)
+    # (appearing in e.g. optax.tree.bias_correction)
     # are promoted (strictly) to float32 when jitted
-    # see https://github.com/google/jax/issues/23337
+    # see https://github.com/jax-ml/jax/issues/23337
     # This may end up letting updates have a dtype different from params.
     # The solution is to fix the dtype of the result to the desired dtype
-    # (just as done in optax.tree_utils.tree_bias_correction).
+    # (just as done in optax.tree.bias_correction).
     # Otherwise, just make sure that all variables defined in the optimizer have
     # the same dtype as the parameters.
     dtype = jnp.dtype(dtype)
@@ -320,7 +410,11 @@ class ContribTest(chex.TestCase):
       update_kwargs = {'value': value}
     else:
       update_kwargs = {}
-    updates, _ = self.variant(opt.update)(grads, state, params, **update_kwargs)
+    if opt_name == 'sophia':
+      update_fn = functools.partial(opt.update, obj_fn=fun)
+    else:
+      update_fn = opt.update
+    updates, _ = self.variant(update_fn)(grads, state, params, **update_kwargs)
     self.assertEqual(updates.dtype, params.dtype)
 
   @chex.variants(
@@ -339,10 +433,16 @@ class ContribTest(chex.TestCase):
     opt = _get_opt_factory(opt_name)(**opt_kwargs)
     if wrapper_name is not None:
       opt = _wrap_opt(opt, wrapper_name, wrapper_kwargs)
-    opt = _accumulation.MultiSteps(opt, every_k_schedule=4)
 
     fun = lambda x: jnp.sum(x**2)
 
+    if opt_name == 'sophia':
+      update_fn = functools.partial(opt.update, obj_fn=fun)
+    else:
+      update_fn = opt.update
+    opt = base.GradientTransformationExtraArgs(opt.init, update_fn)
+    opt = _accumulation.MultiSteps(opt, every_k_schedule=4)
+
     params = jnp.array([1.0, 2.0], dtype=dtype)
     value, grads = jax.value_and_grad(fun)(params)
     state = self.variant(opt.init)(params)
@@ -353,6 +453,98 @@ class ContribTest(chex.TestCase):
     updates, _ = self.variant(opt.update)(grads, state, params, **update_kwargs)
     chex.assert_trees_all_equal(updates, jnp.zeros_like(grads))
 
+  @parameterized.product(
+      _ALL_OPTIMIZERS_UNDER_TEST,
+      dtype=(jnp.float16, jnp.bfloat16, jnp.float32, jnp.float64),
+  )
+  def test_state_shape_dtype_shard_stability(
+      self, opt_name, opt_kwargs, wrapper_name, wrapper_kwargs, dtype):
+    if dtype == jnp.complex128 and jax.default_backend() == 'tpu':
+      self.skipTest('TPU backend does not support complex128')
+    del wrapper_name, wrapper_kwargs  # Unused.
+    with utils.x64_precision(dtype in (jnp.float64, jnp.complex128)):
+      opt = _get_opt_factory(opt_name)(**opt_kwargs)
+      initial_params, _, objective = _setup_parabola(dtype)
+
+      @jax.jit
+      def step(params, state):
+        value, updates = jax.value_and_grad(objective)(params)
+        value = value.astype(jnp.float16 if dtype != jnp.float16
+                             else jnp.bfloat16)  # confuse dtype intentionally
+        if opt_name in ['polyak_sgd', 'momo', 'momo_adam']:
+          update_kwargs = {'value': value}
+        elif opt_name == 'sophia':
+          update_kwargs = {'obj_fn': objective}
+        else:
+          update_kwargs = {}
+        # defeat compiler optimization, use lax.cond to check for stability
+        cond = jax.random.randint(jax.random.key(0), (), 0, 2) == 0
+        updates, state = jax.lax.cond(
+            cond, lambda: opt.update(updates, state, params, **update_kwargs),
+            lambda: (params, state))
+        params = update.apply_updates(params, updates)
+        return params, state
+
+      params = initial_params
+      state = opt.init(params)
+
+      with self.subTest('Test that update is dtype stable'):
+        for _ in range(2):
+          params, new_state = step(params, state)
+          assert jax.tree.leaves(params)[0].dtype == dtype
+          cond = jax.random.randint(jax.random.key(0), (), 0, 2) == 0
+          # pylint: disable=cell-var-from-loop
+          state = jax.lax.cond(cond, lambda: state,  # noqa: B023
+                               lambda: new_state)    # noqa: B023
+          # pylint: enable=cell-var-from-loop
+
+  @parameterized.product(_ALL_OPTIMIZERS_UNDER_TEST)
+  def test_optimizers_accept_learning_rate_schedule_if_type_annotated_as_such(
+      self, opt_name, opt_kwargs, wrapper_name, wrapper_kwargs):
+    del wrapper_kwargs
+    initial_params, _, obj_fn = _setup_parabola(jnp.float32)
+
+    opt_setup = _get_opt_factory(opt_name)
+
+    # check if the optimizer is annotated to accept a learning rate schedule
+    opt_setup_signature = inspect.signature(opt_setup)
+    lr_arg = opt_setup_signature.parameters.get('learning_rate', None)
+    if lr_arg is None or lr_arg.annotation not in (base.ScalarOrSchedule,
+                                                   base.Schedule):
+      self.skipTest('Optimizer doesn\'t accept a learning schedule.')
+    opt_kwargs_with_schedule = dict(
+        opt_kwargs, learning_rate=_schedule.constant_schedule(1e-3))
+    opt_kwargs_with_lr = dict(opt_kwargs, learning_rate=1e-3)
+
+    opt_with_schedule = opt_setup(**opt_kwargs_with_schedule)
+    opt_with_lr = opt_setup(**opt_kwargs_with_lr)
+
+    def step(opt, params, state):
+      value, updates = jax.value_and_grad(obj_fn)(params)
+      if (
+          opt_name in ['momo', 'momo_adam']
+          or wrapper_name == 'reduce_on_plateau'
+      ):
+        update_kwargs = {'value': value}
+      elif opt_name == 'sophia':
+        update_kwargs = {'obj_fn': obj_fn}
+      else:
+        update_kwargs = {}
+      updates, state = opt.update(updates, state, params, **update_kwargs)
+      params = update.apply_updates(params, updates)
+      return params, state
+
+    params_with_schedule, params_with_lr = initial_params, initial_params
+    state_with_schedule = opt_with_schedule.init(params_with_schedule)
+    state_with_lr = opt_with_lr.init(params_with_lr)
+    with self.subTest('Test that optimization works'):
+      for _ in range(10):
+        params_with_schedule, state_with_schedule = step(
+            opt_with_schedule, params_with_schedule, state_with_schedule)
+        params_with_lr, state_with_lr = step(
+            opt_with_lr, params_with_lr, state_with_lr)
+      chex.assert_trees_all_close(params_with_schedule, params_with_lr)
+
 
 if __name__ == '__main__':
   absltest.main()
