@@ -37,7 +37,7 @@ class FenchelYoungTest(chex.TestCase):
   def test_fenchel_young_reg(self):
     # Checks the behavior of the Fenchel-Young loss.
     fy_loss = self.variant(_fenchel_young.make_fenchel_young_loss(logsumexp))
-    rng = jax.random.PRNGKey(0)
+    rng = jax.random.key(0)
     rngs = jax.random.split(rng, 2)
     theta_true = jax.random.uniform(rngs[0], (8, 5))
     y_true = jax.vmap(jax.nn.softmax)(theta_true)
