@@ -152,10 +152,9 @@ def tree_map_params(
   def map_params(maybe_placeholder_value, value):
     if isinstance(maybe_placeholder_value, _ParamsPlaceholder):
       return jax.tree.map(f, value, *rest, is_leaf=is_leaf)
-    elif transform_non_params is not None:
+    if transform_non_params is not None:
       return transform_non_params(value)
-    else:
-      return value
+    return value
 
   return jax.tree.map(
       map_params,
@@ -245,7 +244,7 @@ def tree_get_all_with_path(
     :func:`optax.tree_utils.tree_set`
 
   .. versionadded:: 0.2.2
-  """
+  """  # noqa: E501
   # pylint: enable=line-too-long
   found_values_with_path = _tree_get_all_with_path(tree, key)
   if filtering:
@@ -342,12 +341,14 @@ def tree_get(
 
       >>> params = jnp.array([1., 2., 3.])
       >>> opt = optax.chain(
-      ...     optax.add_noise(1.0, 0.9, 0), optax.scale_by_adam()
+      ...     optax.add_noise(1.0, 0.9, key=0),
+      ...     optax.scale_by_adam()
       ... )
       >>> state = opt.init(params)
       >>> noise_state = optax.tree_utils.tree_get(state, 'AddNoiseState')
       >>> print(noise_state)
-      AddNoiseState(count=Array(0, dtype=int32), rng_key=Array([0, 0], dtype=uint32))
+      AddNoiseState(count=Array(0, dtype=int32), rng_key=Array((), dtype=key<fry>) overlaying:
+      [0 0])
 
     Differentiating between two values by the name of their named tuples.
 
@@ -355,32 +356,24 @@ def tree_get(
       >>> import optax
       >>> params = jnp.array([1., 2., 3.])
       >>> opt = optax.chain(
-      ...   optax.add_noise(1.0, 0.9, 0), optax.scale_by_adam()
+      ...   optax.add_noise(1.0, 0.9, key=0),
+      ...   optax.scale_by_adam()
       ... )
-      >>> state = opt.init(params)
-      >>> filtering = (
-      ...      lambda p, v: isinstance(p[-1], optax.tree_utils.NamedTupleKey)
-      ...      and p[-1].tuple_name == 'ScaleByAdamState'
-      ... )
-      >>> count = optax.tree_utils.tree_get(state, 'count', filtering=filtering)
-      >>> print(count)
-      0
 
   .. seealso:: :func:`optax.tree_utils.tree_get_all_with_path`,
     :func:`optax.tree_utils.tree_set`
 
   .. versionadded:: 0.2.2
-  """
+  """  # noqa: E501
   # pylint: enable=line-too-long
   found_values_with_path = tree_get_all_with_path(
       tree, key, filtering=filtering
   )
   if len(found_values_with_path) > 1:
     raise KeyError(f"Found multiple values for '{key}' in {tree}.")
-  elif not found_values_with_path:
+  if not found_values_with_path:
     return default
-  else:
-    return found_values_with_path[0][1]
+  return found_values_with_path[0][1]
 
 
 def tree_set(
@@ -460,7 +453,7 @@ def tree_set(
     :func:`optax.tree_utils.tree_get`
 
   .. versionadded:: 0.2.2
-  """
+  """  # noqa: E501
   # pylint: enable=line-too-long
 
   # First check if the keys are present in the tree
@@ -472,8 +465,7 @@ def tree_set(
             f"Found no values matching '{key}' given the filtering operation in"
             f" {tree}"
         )
-      else:
-        raise KeyError(f"Found no values matching '{key}' in {tree}")
+      raise KeyError(f"Found no values matching '{key}' in {tree}")
 
   has_any_key = functools.partial(_node_has_keys, keys=tuple(kwargs.keys()))
 
@@ -487,35 +479,30 @@ def tree_set(
       ):
         # The node itself is a named tuple we wanted to replace
         return kwargs[node.__class__.__name__]
-      else:
-        # The node contains one of the keys we want to replace
-        children_with_path = _get_children_with_path(path, node)
-        new_children_with_keys = {}
-        for child_path, child in children_with_path:
-          # Scan each child of that node
-          key = _get_key(child_path[-1])
-          if key in kwargs and (
-              filtering is None or filtering(child_path, child)
-          ):
-            # If the child matches a given key given the filtering operation
-            # replaces with the new value
-            new_children_with_keys.update({key: kwargs[key]})
+      # The node contains one of the keys we want to replace
+      children_with_path = _get_children_with_path(path, node)
+      new_children_with_keys = {}
+      for child_path, child in children_with_path:
+        # Scan each child of that node
+        key = _get_key(child_path[-1])
+        if key in kwargs and (
+            filtering is None or filtering(child_path, child)
+        ):
+          # If the child matches a given key given the filtering operation
+          # replaces with the new value
+          new_children_with_keys.update({key: kwargs[key]})
+        else:
+          if isinstance(child, (dict, list, tuple)):
+            # If the child is itself a pytree, further search in the child to
+            # replace the given value
+            new_children_with_keys.update({key: _replace(child_path, child)})
           else:
-            if (
-                isinstance(child, tuple)
-                or isinstance(child, dict)
-                or isinstance(child, list)
-            ):
-              # If the child is itself a pytree, further search in the child to
-              # replace the given value
-              new_children_with_keys.update({key: _replace(child_path, child)})
-            else:
-              # If the child is just a leaf that does not contain the key or
-              # satisfies the filtering operation, just return the child.
-              new_children_with_keys.update({key: child})
-        return _set_children(node, new_children_with_keys)
-    else:
-      return node
+            # If the child is just a leaf that does not contain the key or
+            # satisfies the filtering operation, just return the child.
+            new_children_with_keys.update({key: child})
+      return _set_children(node, new_children_with_keys)
+
+    return node
 
   # Mimics jax.tree_util.tree_map_with_path(_replace, tree, is_leaf)
   # except that the paths we consider can contain NamedTupleKeys
@@ -667,12 +654,11 @@ def _node_has_keys(node: Any, keys: tuple[Any, ...]) -> bool:
   """
   if _is_named_tuple(node) and any(key in node._fields for key in keys):
     return True
-  elif _is_named_tuple(node) and (node.__class__.__name__ in keys):
+  if _is_named_tuple(node) and (node.__class__.__name__ in keys):
     return True
-  elif isinstance(node, dict) and any(key in node for key in keys):
+  if isinstance(node, dict) and any(key in node for key in keys):
     return True
-  else:
-    return False
+  return False
 
 
 def _flatten_to_key(
@@ -699,10 +685,9 @@ def _flatten_to_key(
     else:
       path_to_key = (*path, NamedTupleKey(node.__class__.__name__, key))
       return (path_to_key, getattr(node, key))
-  elif isinstance(node, dict) and key in node:
+  if isinstance(node, dict) and key in node:
     return (*path, jax.tree_util.DictKey(key)), node[key]
-  else:
-    return path, node
+  return path, node
 
 
 def _get_children_with_path(
@@ -732,15 +717,14 @@ def _get_children_with_path(
         )
         for field in node._fields
     ]
-  elif isinstance(node, dict):
+  if isinstance(node, dict):
     return [
         ((*path, jax.tree_util.DictKey(key)), value)
         for key, value in node.items()
     ]
-  else:
-    raise ValueError(
-        f"Subtree must be a dict or a NamedTuple. Got {type(node)}"
-    )
+  raise ValueError(
+      f"Subtree must be a dict or a NamedTuple. Got {type(node)}"
+  )
 
 
 def _set_children(node: Any, children_with_keys: dict[Any, Any]) -> Any:
@@ -762,12 +746,11 @@ def _set_children(node: Any, children_with_keys: dict[Any, Any]) -> Any:
   """
   if _is_named_tuple(node):
     return node._replace(**children_with_keys)
-  elif isinstance(node, dict):
+  if isinstance(node, dict):
     return children_with_keys
-  else:
-    raise ValueError(
-        f"Subtree must be a dict or a NamedTuple. Got {type(node)}"
-    )
+  raise ValueError(
+      f"Subtree must be a dict or a NamedTuple. Got {type(node)}"
+  )
 
 
 def _get_key(key: _KeyEntry) -> Union[int, str]:
@@ -776,6 +759,7 @@ def _get_key(key: _KeyEntry) -> Union[int, str]:
     if isinstance(key.key, (str, int)):
       return key.key
     raise KeyError("Hashable keys not supported")
+  # pylint: disable=attribute-error
   if isinstance(key, jax.tree_util.FlattenedIndexKey):
     return key.key  # int.
   if isinstance(key, jax.tree_util.GetAttrKey):
@@ -784,4 +768,5 @@ def _get_key(key: _KeyEntry) -> Union[int, str]:
     return key.idx  # int.
   if isinstance(key, NamedTupleKey):
     return key.name  # str.
+  # pylint: enable=attribute-error
   raise KeyError(f"Tree key '{key}' of type '{type(key)}' not valid.")
